#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:ism-cvenum"


class CVEnumISMDissemValuesvalue(Enum):
    """
    :cvar FOUO: FOR OFFICIAL USE ONLY
    :cvar OC: ORIGINATOR CONTROLLED
    :cvar IMC: CONTROLLED IMAGERY
    :cvar SAMI: SOURCES AND METHODS INFORMATION
    :cvar NF: NOT RELEASABLE TO FOREIGN NATIONALS
    :cvar PR: CAUTION-PROPRIETARY INFORMATION INVOLVED
    :cvar REL: AUTHORIZED FOR RELEASE TO
    :cvar RELIDO: RELEASABLE BY INFORMATION DISCLOSURE OFFICIAL
    :cvar RD: RESTRICTED DATA
    :cvar RD_CNWDI: RD-CRITICAL NUCLEAR WEAPON DESIGN INFORMATION
    :cvar FRD: FORMERLY RESTRICTED DATA
    :cvar DCNI: DoD CONTROLLED NUCLEAR INFORMATION
    :cvar UCNI: DoE CONTROLLED NUCLEAR INFORMATION
    :cvar EYES: EYES ONLY
    :cvar DSEN: DEA SENSITIVE
    :cvar FISA: FOREIGN INTELLIGENCE SURVEILLANCE ACT
    """

    FOUO = "FOUO"
    OC = "OC"
    IMC = "IMC"
    SAMI = "SAMI"
    NF = "NF"
    PR = "PR"
    REL = "REL"
    RELIDO = "RELIDO"
    RD = "RD"
    RD_CNWDI = "RD-CNWDI"
    FRD = "FRD"
    DCNI = "DCNI"
    UCNI = "UCNI"
    EYES = "EYES"
    DSEN = "DSEN"
    FISA = "FISA"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from dataclasses import dataclass, field
from typing import List, Optional, Union

from xsdata.models.datatype import XmlDate

from ..ismcat.cvegenerated.cvenum_ismcatfgiopen import CVEnumISMCATFGIOpenValuesvalue
from ..ismcat.cvegenerated.cvenum_ismcatfgiprotected import CVEnumISMCATFGIProtectedValuesvalue
from ..ismcat.cvegenerated.cvenum_ismcatowner_producer import CVEnumISMCATOwnerProducerValuesvalue
from ..ismcat.cvegenerated.cvenum_ismcatrel_to import CVEnumISMCATRelToValuesvalue
from .cvegenerated.cvenum_ism25_x import CVEnumISM25X
from .cvegenerated.cvenum_ismatomic_energy_markings import CVEnumISMatomicEnergyMarkingsValuesvalue
from .cvegenerated.cvenum_ismclassification_all import CVEnumISMClassificationAll
from .cvegenerated.cvenum_ismdissem import CVEnumISMDissemValues
from .cvegenerated.cvenum_ismnon_ic import CVEnumISMNonICValuesvalue
from .cvegenerated.cvenum_ismnon_uscontrols import CVEnumISMNonUSControlsValues
from .cvegenerated.cvenum_ismnotice import CVEnumISMNoticeValues
from .cvegenerated.cvenum_ismpoc_type import CVEnumISMPocTypeValues
from .cvegenerated.cvenum_ismscicontrols import CVEnumISMSCIControlsValuesvalue

__NAMESPACE__ = "urn:us:gov:ic:ism:13"


@dataclass
class LongStringWithSecurityType:
    value: str = field(
        default="",
        metadata={
            "required": True,
            "max_length": 32000,
        },
    )
    classification: Optional[CVEnumISMClassificationAll] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
        },
    )
    owner_producer: List[Union[str, CVEnumISMCATOwnerProducerValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "ownerProducer",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    joint: Optional[bool] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    scicontrols: List[Union[str, CVEnumISMSCIControlsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "SCIcontrols",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"KDK-BLFH-[A-Z0-9]{1,6}|KDK-IDIT-[A-Z0-9]{1,6}|KDK-KAND-[A-Z0-9]{1,6}|RSV-[A-Z0-9]{3}|SI-G-[A-Z]{4}|SI-[A-Z]{3}|SI-[A-Z]{3}-[A-Z]{4}",
            "tokens": True,
        },
    )
    saridentifier: List[str] = field(
        default_factory=list,
        metadata={
            "name": "SARIdentifier",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"[A-Z_0-9\-]{1,100}|[A-Z]{2,}|[A-Z]{2,}-[A-Z][A-Z0-9]+|[A-Z]{2,}-[A-Z][A-Z0-9]+-[A-Z0-9]{2,}",
            "tokens": True,
        },
    )
    atomic_energy_markings: List[Union[str, CVEnumISMatomicEnergyMarkingsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "atomicEnergyMarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"RD-SG-((14)|(15)|(18)|(20))|FRD-SG-((14)|(15)|(18)|(20))",
            "tokens": True,
        },
    )
    dissemination_controls: List[CVEnumISMDissemValues] = field(
        default_factory=list,
        metadata={
            "name": "disseminationControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    display_only_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "displayOnlyTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_open: List[Union[str, CVEnumISMCATFGIOpenValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceOpen",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_protected: List[Union[str, CVEnumISMCATFGIProtectedValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceProtected",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    releasable_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "releasableTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    non_icmarkings: List[Union[str, CVEnumISMNonICValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "nonICmarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"ACCM-[A-Z0-9\-_]{1,61}|NNPI",
            "tokens": True,
        },
    )
    classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "classifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    compilation_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "compilationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    derivatively_classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivativelyClassifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    classification_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "classificationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 4096,
        },
    )
    non_uscontrols: List[CVEnumISMNonUSControlsValues] = field(
        default_factory=list,
        metadata={
            "name": "nonUSControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    derived_from: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivedFrom",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "declassDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    declass_event: Optional[str] = field(
        default=None,
        metadata={
            "name": "declassEvent",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_exception: Optional[CVEnumISM25X] = field(
        default=None,
        metadata={
            "name": "declassException",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )


@dataclass
class ShortStringWithSecurityType:
    value: str = field(
        default="",
        metadata={
            "required": True,
            "max_length": 256,
        },
    )
    classification: Optional[CVEnumISMClassificationAll] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
        },
    )
    owner_producer: List[Union[str, CVEnumISMCATOwnerProducerValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "ownerProducer",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    joint: Optional[bool] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    scicontrols: List[Union[str, CVEnumISMSCIControlsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "SCIcontrols",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"KDK-BLFH-[A-Z0-9]{1,6}|KDK-IDIT-[A-Z0-9]{1,6}|KDK-KAND-[A-Z0-9]{1,6}|RSV-[A-Z0-9]{3}|SI-G-[A-Z]{4}|SI-[A-Z]{3}|SI-[A-Z]{3}-[A-Z]{4}",
            "tokens": True,
        },
    )
    saridentifier: List[str] = field(
        default_factory=list,
        metadata={
            "name": "SARIdentifier",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"[A-Z_0-9\-]{1,100}|[A-Z]{2,}|[A-Z]{2,}-[A-Z][A-Z0-9]+|[A-Z]{2,}-[A-Z][A-Z0-9]+-[A-Z0-9]{2,}",
            "tokens": True,
        },
    )
    atomic_energy_markings: List[Union[str, CVEnumISMatomicEnergyMarkingsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "atomicEnergyMarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"RD-SG-((14)|(15)|(18)|(20))|FRD-SG-((14)|(15)|(18)|(20))",
            "tokens": True,
        },
    )
    dissemination_controls: List[CVEnumISMDissemValues] = field(
        default_factory=list,
        metadata={
            "name": "disseminationControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    display_only_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "displayOnlyTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_open: List[Union[str, CVEnumISMCATFGIOpenValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceOpen",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_protected: List[Union[str, CVEnumISMCATFGIProtectedValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceProtected",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    releasable_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "releasableTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    non_icmarkings: List[Union[str, CVEnumISMNonICValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "nonICmarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"ACCM-[A-Z0-9\-_]{1,61}|NNPI",
            "tokens": True,
        },
    )
    classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "classifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    compilation_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "compilationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    derivatively_classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivativelyClassifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    classification_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "classificationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 4096,
        },
    )
    non_uscontrols: List[CVEnumISMNonUSControlsValues] = field(
        default_factory=list,
        metadata={
            "name": "nonUSControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    derived_from: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivedFrom",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "declassDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    declass_event: Optional[str] = field(
        default=None,
        metadata={
            "name": "declassEvent",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_exception: Optional[CVEnumISM25X] = field(
        default=None,
        metadata={
            "name": "declassException",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )


@dataclass
class NoticeText(LongStringWithSecurityType):
    """
    <p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">The actual text of a notice.</p>
    """

    class Meta:
        namespace = "urn:us:gov:ic:ism:13"

    poc_type: List[CVEnumISMPocTypeValues] = field(
        default_factory=list,
        metadata={
            "name": "pocType",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )


@dataclass
class NoticeBaseType:
    """<p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">Base type for Notices.

    Does not include any attributes.</p>
    """

    notice_text: List[NoticeText] = field(
        default_factory=list,
        metadata={
            "name": "NoticeText",
            "type": "Element",
            "namespace": "urn:us:gov:ic:ism:13",
            "min_occurs": 1,
        },
    )


@dataclass
class NoticeExternalType(NoticeBaseType):
    """
    <p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">A single Notice that may consist of 1 or more NoticeText
    for use when the notice refers to something external.</p>
    """

    notice_type: List[CVEnumISMNoticeValues] = field(
        default_factory=list,
        metadata={
            "name": "noticeType",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    notice_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "noticeReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 2048,
        },
    )
    notice_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "noticeDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    unregistered_notice_type: Optional[str] = field(
        default=None,
        metadata={
            "name": "unregisteredNoticeType",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 2048,
        },
    )
    external_notice: bool = field(
        init=False,
        default=True,
        metadata={
            "name": "externalNotice",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
        },
    )
    classification: Optional[CVEnumISMClassificationAll] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    owner_producer: List[Union[str, CVEnumISMCATOwnerProducerValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "ownerProducer",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    joint: Optional[bool] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    scicontrols: List[Union[str, CVEnumISMSCIControlsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "SCIcontrols",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"KDK-BLFH-[A-Z0-9]{1,6}|KDK-IDIT-[A-Z0-9]{1,6}|KDK-KAND-[A-Z0-9]{1,6}|RSV-[A-Z0-9]{3}|SI-G-[A-Z]{4}|SI-[A-Z]{3}|SI-[A-Z]{3}-[A-Z]{4}",
            "tokens": True,
        },
    )
    saridentifier: List[str] = field(
        default_factory=list,
        metadata={
            "name": "SARIdentifier",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"[A-Z_0-9\-]{1,100}|[A-Z]{2,}|[A-Z]{2,}-[A-Z][A-Z0-9]+|[A-Z]{2,}-[A-Z][A-Z0-9]+-[A-Z0-9]{2,}",
            "tokens": True,
        },
    )
    atomic_energy_markings: List[Union[str, CVEnumISMatomicEnergyMarkingsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "atomicEnergyMarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"RD-SG-((14)|(15)|(18)|(20))|FRD-SG-((14)|(15)|(18)|(20))",
            "tokens": True,
        },
    )
    dissemination_controls: List[CVEnumISMDissemValues] = field(
        default_factory=list,
        metadata={
            "name": "disseminationControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    display_only_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "displayOnlyTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_open: List[Union[str, CVEnumISMCATFGIOpenValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceOpen",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_protected: List[Union[str, CVEnumISMCATFGIProtectedValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceProtected",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    releasable_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "releasableTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    non_icmarkings: List[Union[str, CVEnumISMNonICValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "nonICmarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"ACCM-[A-Z0-9\-_]{1,61}|NNPI",
            "tokens": True,
        },
    )
    classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "classifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    compilation_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "compilationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    derivatively_classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivativelyClassifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    classification_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "classificationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 4096,
        },
    )
    non_uscontrols: List[CVEnumISMNonUSControlsValues] = field(
        default_factory=list,
        metadata={
            "name": "nonUSControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    derived_from: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivedFrom",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "declassDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    declass_event: Optional[str] = field(
        default=None,
        metadata={
            "name": "declassEvent",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_exception: Optional[CVEnumISM25X] = field(
        default=None,
        metadata={
            "name": "declassException",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )


@dataclass
class NoticeType(NoticeBaseType):
    """
    <p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">A single Notice that may consist of 1 or more
    NoticeText</p>
    """

    notice_type: List[CVEnumISMNoticeValues] = field(
        default_factory=list,
        metadata={
            "name": "noticeType",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    notice_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "noticeReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 2048,
        },
    )
    notice_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "noticeDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    unregistered_notice_type: Optional[str] = field(
        default=None,
        metadata={
            "name": "unregisteredNoticeType",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 2048,
        },
    )
    external_notice: Optional[bool] = field(
        default=None,
        metadata={
            "name": "externalNotice",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    classification: Optional[CVEnumISMClassificationAll] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    owner_producer: List[Union[str, CVEnumISMCATOwnerProducerValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "ownerProducer",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    joint: Optional[bool] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    scicontrols: List[Union[str, CVEnumISMSCIControlsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "SCIcontrols",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"KDK-BLFH-[A-Z0-9]{1,6}|KDK-IDIT-[A-Z0-9]{1,6}|KDK-KAND-[A-Z0-9]{1,6}|RSV-[A-Z0-9]{3}|SI-G-[A-Z]{4}|SI-[A-Z]{3}|SI-[A-Z]{3}-[A-Z]{4}",
            "tokens": True,
        },
    )
    saridentifier: List[str] = field(
        default_factory=list,
        metadata={
            "name": "SARIdentifier",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"[A-Z_0-9\-]{1,100}|[A-Z]{2,}|[A-Z]{2,}-[A-Z][A-Z0-9]+|[A-Z]{2,}-[A-Z][A-Z0-9]+-[A-Z0-9]{2,}",
            "tokens": True,
        },
    )
    atomic_energy_markings: List[Union[str, CVEnumISMatomicEnergyMarkingsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "atomicEnergyMarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"RD-SG-((14)|(15)|(18)|(20))|FRD-SG-((14)|(15)|(18)|(20))",
            "tokens": True,
        },
    )
    dissemination_controls: List[CVEnumISMDissemValues] = field(
        default_factory=list,
        metadata={
            "name": "disseminationControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    display_only_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "displayOnlyTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_open: List[Union[str, CVEnumISMCATFGIOpenValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceOpen",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_protected: List[Union[str, CVEnumISMCATFGIProtectedValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceProtected",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    releasable_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "releasableTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    non_icmarkings: List[Union[str, CVEnumISMNonICValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "nonICmarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"ACCM-[A-Z0-9\-_]{1,61}|NNPI",
            "tokens": True,
        },
    )
    classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "classifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    compilation_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "compilationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    derivatively_classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivativelyClassifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    classification_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "classificationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 4096,
        },
    )
    non_uscontrols: List[CVEnumISMNonUSControlsValues] = field(
        default_factory=list,
        metadata={
            "name": "nonUSControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    derived_from: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivedFrom",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "declassDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    declass_event: Optional[str] = field(
        default=None,
        metadata={
            "name": "declassEvent",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_exception: Optional[CVEnumISM25X] = field(
        default=None,
        metadata={
            "name": "declassException",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )


@dataclass
class Notice(NoticeType):
    """
    <p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">A single Notice that may consist of 1 or more
    NoticeText</p>
    """

    class Meta:
        namespace = "urn:us:gov:ic:ism:13"


@dataclass
class NoticeExternal(NoticeExternalType):
    """
    <p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">A single Notice that may consist of 1 or more NoticeText
    for use when the notice refers to something external.</p>
    """

    class Meta:
        namespace = "urn:us:gov:ic:ism:13"


@dataclass
class NoticeExternalListType:
    """
    <p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">A list of Notices</p>
    """

    notice_external: List[NoticeExternal] = field(
        default_factory=list,
        metadata={
            "name": "NoticeExternal",
            "type": "Element",
            "namespace": "urn:us:gov:ic:ism:13",
            "min_occurs": 1,
        },
    )
    classification: Optional[CVEnumISMClassificationAll] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
        },
    )
    owner_producer: List[Union[str, CVEnumISMCATOwnerProducerValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "ownerProducer",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    joint: Optional[bool] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    scicontrols: List[Union[str, CVEnumISMSCIControlsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "SCIcontrols",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"KDK-BLFH-[A-Z0-9]{1,6}|KDK-IDIT-[A-Z0-9]{1,6}|KDK-KAND-[A-Z0-9]{1,6}|RSV-[A-Z0-9]{3}|SI-G-[A-Z]{4}|SI-[A-Z]{3}|SI-[A-Z]{3}-[A-Z]{4}",
            "tokens": True,
        },
    )
    saridentifier: List[str] = field(
        default_factory=list,
        metadata={
            "name": "SARIdentifier",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"[A-Z_0-9\-]{1,100}|[A-Z]{2,}|[A-Z]{2,}-[A-Z][A-Z0-9]+|[A-Z]{2,}-[A-Z][A-Z0-9]+-[A-Z0-9]{2,}",
            "tokens": True,
        },
    )
    atomic_energy_markings: List[Union[str, CVEnumISMatomicEnergyMarkingsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "atomicEnergyMarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"RD-SG-((14)|(15)|(18)|(20))|FRD-SG-((14)|(15)|(18)|(20))",
            "tokens": True,
        },
    )
    dissemination_controls: List[CVEnumISMDissemValues] = field(
        default_factory=list,
        metadata={
            "name": "disseminationControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    display_only_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "displayOnlyTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_open: List[Union[str, CVEnumISMCATFGIOpenValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceOpen",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_protected: List[Union[str, CVEnumISMCATFGIProtectedValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceProtected",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    releasable_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "releasableTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    non_icmarkings: List[Union[str, CVEnumISMNonICValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "nonICmarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"ACCM-[A-Z0-9\-_]{1,61}|NNPI",
            "tokens": True,
        },
    )
    classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "classifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    compilation_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "compilationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    derivatively_classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivativelyClassifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    classification_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "classificationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 4096,
        },
    )
    non_uscontrols: List[CVEnumISMNonUSControlsValues] = field(
        default_factory=list,
        metadata={
            "name": "nonUSControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    derived_from: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivedFrom",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "declassDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    declass_event: Optional[str] = field(
        default=None,
        metadata={
            "name": "declassEvent",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_exception: Optional[CVEnumISM25X] = field(
        default=None,
        metadata={
            "name": "declassException",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )


@dataclass
class NoticeListType:
    """
    <p xmlns:ns1="urn:us:gov:ic:ism:13" ns1:ownerProducer="USA"
    ns1:classification="U">A list of Notices</p>
    """

    notice: List[Notice] = field(
        default_factory=list,
        metadata={
            "name": "Notice",
            "type": "Element",
            "namespace": "urn:us:gov:ic:ism:13",
            "min_occurs": 1,
        },
    )
    classification: Optional[CVEnumISMClassificationAll] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
        },
    )
    owner_producer: List[Union[str, CVEnumISMCATOwnerProducerValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "ownerProducer",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "required": True,
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    joint: Optional[bool] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    scicontrols: List[Union[str, CVEnumISMSCIControlsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "SCIcontrols",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"KDK-BLFH-[A-Z0-9]{1,6}|KDK-IDIT-[A-Z0-9]{1,6}|KDK-KAND-[A-Z0-9]{1,6}|RSV-[A-Z0-9]{3}|SI-G-[A-Z]{4}|SI-[A-Z]{3}|SI-[A-Z]{3}-[A-Z]{4}",
            "tokens": True,
        },
    )
    saridentifier: List[str] = field(
        default_factory=list,
        metadata={
            "name": "SARIdentifier",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"[A-Z_0-9\-]{1,100}|[A-Z]{2,}|[A-Z]{2,}-[A-Z][A-Z0-9]+|[A-Z]{2,}-[A-Z][A-Z0-9]+-[A-Z0-9]{2,}",
            "tokens": True,
        },
    )
    atomic_energy_markings: List[Union[str, CVEnumISMatomicEnergyMarkingsValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "atomicEnergyMarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"RD-SG-((14)|(15)|(18)|(20))|FRD-SG-((14)|(15)|(18)|(20))",
            "tokens": True,
        },
    )
    dissemination_controls: List[CVEnumISMDissemValues] = field(
        default_factory=list,
        metadata={
            "name": "disseminationControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    display_only_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "displayOnlyTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_open: List[Union[str, CVEnumISMCATFGIOpenValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceOpen",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    fgisource_protected: List[Union[str, CVEnumISMCATFGIProtectedValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "FGIsourceProtected",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    releasable_to: List[Union[str, CVEnumISMCATRelToValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "releasableTo",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"NATO/[a-zA-Z\-_]",
            "tokens": True,
        },
    )
    non_icmarkings: List[Union[str, CVEnumISMNonICValuesvalue]] = field(
        default_factory=list,
        metadata={
            "name": "nonICmarkings",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "pattern": r"ACCM-[A-Z0-9\-_]{1,61}|NNPI",
            "tokens": True,
        },
    )
    classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "classifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    compilation_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "compilationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    derivatively_classified_by: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivativelyClassifiedBy",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    classification_reason: Optional[str] = field(
        default=None,
        metadata={
            "name": "classificationReason",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 4096,
        },
    )
    non_uscontrols: List[CVEnumISMNonUSControlsValues] = field(
        default_factory=list,
        metadata={
            "name": "nonUSControls",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "tokens": True,
        },
    )
    derived_from: Optional[str] = field(
        default=None,
        metadata={
            "name": "derivedFrom",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_date: Optional[XmlDate] = field(
        default=None,
        metadata={
            "name": "declassDate",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )
    declass_event: Optional[str] = field(
        default=None,
        metadata={
            "name": "declassEvent",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
            "max_length": 1024,
        },
    )
    declass_exception: Optional[CVEnumISM25X] = field(
        default=None,
        metadata={
            "name": "declassException",
            "type": "Attribute",
            "namespace": "urn:us:gov:ic:ism:13",
        },
    )


@dataclass
class NoticeExternalList(NoticeExternalListType):
    class Meta:
        namespace = "urn:us:gov:ic:ism:13"


@dataclass
class NoticeList(NoticeListType):
    class Meta:
        namespace = "urn:us:gov:ic:ism:13"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
from math import radians
from typing import Optional
from xml.etree import ElementTree as ET

from aws.osml.photogrammetry import GeodeticWorldCoordinate, ImageCoordinate, ProjectiveSensorModel

from .sensor_model_builder import SensorModelBuilder
from .xmltre_utils import get_tre_field_value


class ProjectiveSensorModelBuilder(SensorModelBuilder):
    """
    This builder is used to create sensor models for images that have an IGEOLO or CSCRNA TREs. The inputs are not the
    TREs themselves but rather the XML formatted TREs resulting from GDAL's parsing of that information. This
    information can be obtained by accessing the xml:TRE metadata domain for a given GDAL dataset.

    See STDI-0002 Volume 3 Appendix B for more detailed information.
    """

    def __init__(self, xml_tres: ET.Element, full_image_width: float, full_image_height: float) -> None:
        """
        Constructor for the builder accepting the required XML TREs and image dimensions.

        :param xml_tres: the XML tres for this image
        :param full_image_width: the width of the image in pixels
        :param full_image_height: the height of the image in pixels

        :return: None
        """
        super().__init__()
        self.xml_tres = xml_tres
        self.full_image_width = full_image_width
        self.full_image_height = full_image_height

    def build(self) -> Optional[ProjectiveSensorModel]:
        """
        Examine the TRE metadata for corner coordinate information, parse the necessary values out of those TREs,
        and construct a projective sensor model.

        :return: a ProjectiveSensorModel if one can be constructed, None otherwise
        """

        # Check to see if an CSCRNA TRE is included with the metadata. It would contain the necessary parameters for
        # this type of sensor model.
        cscrna_tre = self.xml_tres.find("./tre[@name='CSCRNA']")
        if cscrna_tre is None:
            logging.debug("No CSCRNA TRE found.")
            return None

        return ProjectiveSensorModelBuilder.build_projective_sensor_model(
            cscrna_tre, self.full_image_width, self.full_image_height
        )

    @staticmethod
    def build_projective_sensor_model(
        cscrna_tre: ET.Element, full_image_width: float, full_image_height: float
    ) -> ProjectiveSensorModel:
        """
        This private method constructs an RPC sensor model from an RPC00B TRE.

        :param cscrna_tre: the GDAL XML for CSCRNA
        :param full_image_width: the width of the image in pixels
        :param full_image_height: the height of the image in pixels

        :return: the projective sensor model
        """

        world_coordinates = [
            GeodeticWorldCoordinate(
                [
                    radians(get_tre_field_value(cscrna_tre, f"{corner}CNR_LONG", float)),
                    radians(get_tre_field_value(cscrna_tre, f"{corner}CNR_LAT", float)),
                    radians(get_tre_field_value(cscrna_tre, f"{corner}CNR_HT", float)),
                ]
            )
            for corner in ["UL", "UR", "LR", "LL"]
        ]
        image_coordinates = [
            ImageCoordinate([0, 0]),
            ImageCoordinate([full_image_width, 0]),
            ImageCoordinate([full_image_width, full_image_height]),
            ImageCoordinate([0, full_image_height]),
        ]
        return ProjectiveSensorModel(world_coordinates, image_coordinates)



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
from typing import Dict, Optional

from osgeo import gdal

logger = logging.getLogger(__name__)


def set_gdal_default_configuration() -> None:
    """
    This function sets GDAL configuration options to support efficient reading of large raster
    datasets using the /vsis3 virtual file system.

    :return: None
    """
    # This is the maximum size of a chunk we can fetch at one time from a remote file
    # I couldn't find the value anywhere in the documentation, but it is enforced here:
    # https://github.com/OSGeo/gdal/blob/211e2430b8cda486d0e0e68446647f56cc0ca149/port/cpl_vsil_curl.cpp#L161
    max_curl_chunk_size = 10 * 1024 * 1024

    # For information on these options and their usage please see:
    # https://gdal.org/user/configoptions.html
    gdal_default_environment_options = {
        "VSI_CACHE": "YES",
        "VSI_CACHE_SIZE": "10000000000",
        "GDAL_DISABLE_READDIR_ON_OPEN": "EMPTY_DIR",
        "GDAL_CACHEMAX": "75%",
        "GDAL_NUM_THREADS": "1",
        "GDAL_GCPS_TO_GEOTRANSFORM_APPROX_OK": "YES",
        "GDAL_MAX_DATASET_POOL_SIZE": "1000",
        "BIGTIFF_OVERVIEW": "YES",
        "USE_TILE_AS_BLOCK": "YES",
        # This flag will setup verbose output for GDAL. In particular, it will show you each range
        # read for the file if using the /vsis3 virtual file system.
        "CPL_DEBUG": "OFF",
        "CPL_VSIL_CURL_CHUNK_SIZE": str(max_curl_chunk_size),
        "CPL_VSIL_CURL_CACHE_SIZE": str(max_curl_chunk_size * 500),
    }
    for key, val in gdal_default_environment_options.items():
        gdal.SetConfigOption(key, str(val))
    logger.info("Set GDAL Configuration Options: {}".format(gdal_default_environment_options))


class GDALConfigEnv:
    """
    This class provides a way to setup a temporary GDAL environment using Python's "with"
    statement. GDAL configuration options will be set inside the scope of the with statement and
    then reverted to previously set values on exit. This will commonly be used to set AWS security
    credentials (e.g. AWS_SECRET_ACCESS_KEY) for use by other GDAL operations.

    See: https://gdal.org/user/configoptions.html#gdal-configuration-file for additional options.
    """

    def __init__(self, options: Dict = None) -> None:
        if options:
            self.options = options.copy()
        else:
            self.options = {}
        self.old_options: Dict = {}

    def with_aws_credentials(self, aws_credentials: Optional[Dict[str, str]]) -> "GDALConfigEnv":
        """
        This method sets the GDAL configuration options for the AWS credentials from the
        credentials object returned by a boto3 call to sts.assume_role(...).

        :param aws_credentials: the dictionary of values from the sts.assume_role() response['Credentials']
        :return: self to facilitate a simple builder constructor pattern
        """
        if aws_credentials is not None:
            self.options.update(
                {
                    "AWS_SECRET_ACCESS_KEY": aws_credentials["SecretAccessKey"],
                    "AWS_ACCESS_KEY_ID": aws_credentials["AccessKeyId"],
                    "AWS_SESSION_TOKEN": aws_credentials["SessionToken"],
                }
            )
        return self

    def __enter__(self) -> None:
        for key, val in self.options.items():
            self.old_options[key] = gdal.GetConfigOption(key)
            gdal.SetConfigOption(key, str(val))

    def __exit__(self, exc_type: str, exc_val: str, exc_traceback: str) -> None:
        for key, val in self.options.items():
            gdal.SetConfigOption(key, self.old_options[key])



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:ism-cvenum"


class CVEnumISMSourceMarked(Enum):
    """(U) All currently authorized Source Marked values
    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMSourceMarked.xml

    :cvar OADR: Source Marked OADR (Originating Agency's Determination Required)
    :cvar X1: Source Marked X1
    :cvar X2: Source Marked X2
    :cvar X3: Source Marked X3
    :cvar X4: Source Marked X4
    :cvar X5: Source Marked X5
    :cvar X6: Source Marked X6
    :cvar X7: Source Marked X7
    :cvar X8: Source Marked X8
    """

    OADR = "OADR"
    X1 = "X1"
    X2 = "X2"
    X3 = "X3"
    X4 = "X4"
    X5 = "X5"
    X6 = "X6"
    X7 = "X7"
    X8 = "X8"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:cvenum:ism:notice"


class CVEnumISMNoticeValues(Enum):
    """(U) All currently authorized Notice values
    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMNotice.xml

    :cvar FISA: FISA Warning statement
    :cvar IMC: IMCON Warning statement
    :cvar CNWDI: Controled Nuclear Weapon Design Information Warning statement
    :cvar RD: RD Warning statement
    :cvar FRD: FRD Warning statement
    :cvar DS: LIMDIS caveat
    :cvar LES: LES Notice
    :cvar LES_NF: LES-NF Notice
    :cvar DSEN: DSEN Notice
    :cvar DO_D_DIST_A: DoD Distribution statement A from DoD Directive 5230.24
    :cvar DO_D_DIST_B: DoD Distribution statement B from DoD Directive 5230.24
    :cvar DO_D_DIST_C: DoD Distribution statement C from DoD Directive 5230.24
    :cvar DO_D_DIST_D: DoD Distribution statement D from DoD Directive 5230.24
    :cvar DO_D_DIST_E: DoD Distribution statement E from DoD Directive 5230.24
    :cvar DO_D_DIST_F: DoD Distribution statement F from DoD Directive 5230.24
    :cvar DO_D_DIST_X: DoD Distribution statement X from DoD Directive 5230.24
    :cvar US_PERSON: US Person info Notice
    :cvar PRE13526_ORCON: Indicates that an instance document must abide by rules pertaining to ORIGINATOR
        CONTROLLED data issued prior to Executive Order 13526.
    :cvar POC: Indicates that the contents of this notice specify the contact information for a required point-
        of-contact.
    :cvar COMSEC: COMSEC Notice
    """

    FISA = "FISA"
    IMC = "IMC"
    CNWDI = "CNWDI"
    RD = "RD"
    FRD = "FRD"
    DS = "DS"
    LES = "LES"
    LES_NF = "LES-NF"
    DSEN = "DSEN"
    DO_D_DIST_A = "DoD-Dist-A"
    DO_D_DIST_B = "DoD-Dist-B"
    DO_D_DIST_C = "DoD-Dist-C"
    DO_D_DIST_D = "DoD-Dist-D"
    DO_D_DIST_E = "DoD-Dist-E"
    DO_D_DIST_F = "DoD-Dist-F"
    DO_D_DIST_X = "DoD-Dist-X"
    US_PERSON = "US-Person"
    PRE13526_ORCON = "pre13526ORCON"
    POC = "POC"
    COMSEC = "COMSEC"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

from math import radians
from typing import List, Optional

from osgeo import gdal

from aws.osml.photogrammetry import GDALAffineSensorModel, GeodeticWorldCoordinate, ImageCoordinate, ProjectiveSensorModel

from .sensor_model_builder import SensorModelBuilder


class GDALAffineSensorModelBuilder(SensorModelBuilder):
    """
    This builder is used to create sensor models for images that have GDAL geo transforms.
    """

    def __init__(self, geo_transform: List[float], proj_wkt: Optional[str] = None) -> None:
        """
        Constructor for the builder accepting the required GDAL geotransform.

        :param geo_transform: the geotransform for this image
        :param proj_wkt: the well known text string of the CRS used by the image

        :return: None
        """
        super().__init__()
        self.geo_transform = geo_transform
        self.proj_wkt = proj_wkt

    def build(self) -> Optional[GDALAffineSensorModel]:
        """
        Use the GDAL GeoTransform to construct a sensor model.

        :return: affine transform based SensorModel that uses the GDAL GeoTransform
        """
        if self.geo_transform is None:
            return None
        return GDALAffineSensorModel(self.geo_transform, self.proj_wkt)


class GDALGCPSensorModelBuilder(SensorModelBuilder):
    """
    This builder is used to create sensor models for images that have GDAL ground control points (GCPs).
    """

    def __init__(self, ground_control_points: List[gdal.GCP]) -> None:
        """
        Constructor for the builder accepting the required GDAL GCPs.

        :param ground_control_points: the ground control points for this image

        :return: None
        """
        super().__init__()
        self.ground_control_points = ground_control_points

    def build(self) -> Optional[ProjectiveSensorModel]:
        """
        Use the GCPs to construct a projective sensor model.

        :return: a projective transform SensorModel that uses the GDAL GCPs provided
        """
        if not self.ground_control_points or len(self.ground_control_points) < 4:
            return None

        world_coordinates = [
            GeodeticWorldCoordinate([radians(gcp.GCPX), radians(gcp.GCPY), gcp.GCPZ]) for gcp in self.ground_control_points
        ]
        image_coordinates = [ImageCoordinate([gcp.GCPPixel, gcp.GCPLine]) for gcp in self.ground_control_points]
        return ProjectiveSensorModel(world_coordinates, image_coordinates)



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:ism-cvenum"


class CVEnumISMClassificationAll(Enum):
    """(U) All currently valid classification marks
    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMClassificationAll.xml

    :cvar R: RESTRICTED
    :cvar C: CONFIDENTIAL
    :cvar S: SECRET
    :cvar TS: TOP SECRET
    :cvar U: UNCLASSIFIED
    """

    R = "R"
    C = "C"
    S = "S"
    TS = "TS"
    U = "U"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
from math import floor
from typing import List, Optional, Tuple

from xsdata.formats.dataclass.parsers import XmlParser
from xsdata.formats.dataclass.serializers import XmlSerializer
from xsdata.formats.dataclass.serializers.config import SerializerConfig

logger = logging.getLogger(__name__)


class SICDUpdater:
    """
    This class provides a means to perform common updates to a SICD XML metadata document.
    """

    def __init__(self, xml_str: str):
        """
        Construct a new instance of this class to manage a given set of SICD metadata.

        :param xml_str: the SICD XML metadata to update
        """
        self.xml_str = xml_str
        if self.xml_str is not None and len(self.xml_str) > 0:
            parser = XmlParser()
            self.sicd = parser.from_string(self.xml_str)

        # Here we're storing off the original first row/col to support the case where multiple chips are
        # created from a SICD image that has already been chipped.
        self.original_first_row = self.sicd.image_data.first_row
        self.original_first_col = self.sicd.image_data.first_col

    def update_image_data_for_chip(self, chip_bounds: List[int], output_size: Optional[Tuple[int, int]]) -> None:
        """
        This updates the SICD ImageData structure so that the FirstRow, FirstCol and NumRows, NumCols
        elements match the new chip boundary.A sample of this XML structure is shown below::

            <ImageData>
                <NumRows>3859</NumRows>
                <NumCols>6679</NumCols>
                <FirstRow>0</FirstRow>
                <FirstCol>0</FirstCol>
                <FullImage>
                    <NumRows>3859</NumRows>
                    <NumCols>6679</NumCols>
                </FullImage>
            </ImageData>


        :param chip_bounds: the [col, row, width, height] of the chip boundary
        :param output_size: the [width, height] of the output chip
        """

        if output_size is not None and (output_size[0] != chip_bounds[2] or output_size[1] != chip_bounds[3]):
            raise ValueError("SICD chipping does not support scaling operations.")

        self.sicd.image_data.first_row = floor(float(self.original_first_row)) + int(chip_bounds[1])
        self.sicd.image_data.first_col = floor(float(self.original_first_col)) + int(chip_bounds[0])
        self.sicd.image_data.num_rows = int(chip_bounds[3])
        self.sicd.image_data.num_cols = int(chip_bounds[2])

    def encode_current_xml(self) -> str:
        """
        Returns a copy of the current SICD metadata encoded in XML.

        :return: xml encoded SICD metadata
        """
        serializer = XmlSerializer(config=SerializerConfig(pretty_print=False))
        updated_xml = serializer.render(self.sicd)
        return updated_xml



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:cvenum:ism:25x"


class CVEnumISM25X(Enum):
    """(U) All currently authorized authority block declass date/event exemptions.

    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISM25X.xml

    :cvar AEA: When using a source document that contains portions of Restricted Data (RD) or Formerly Restricted
        Data (FRD) where the RD/FRD source document(s) do not have declassification instructions, the
        derivatively classified document shall not contain a declassification date or event on the Declassify On
        line.  The following shall be annotated on the Declassify On line:  "Not Applicable or (N/A) to RD/FRD
        portions" and "See source list for NSI portions" separated by a period. The source list must include the
        declassification instruction for each of the source documents classified under E.O. 13526 and shall not
        appear in the classification authority block
    :cvar NATO: Since NATO information is not to be declassified or downgraded without the prior consent of NATO,
        the “Declassify on” line of documents that commingle information classified by NATO and U.S. classified
        NSI, will read “N/A to NATO portions. See source list for NSI portions.” The NSI source list will appear
        beneath the classification authority block in a manner that clearly identifies it as separate and
        distinct.
    :cvar NATO_AEA: Handles special case of BOTH NATO and AEA as a single exemption.
    :cvar VALUE_25_X1: Reveal the identity of a confidential human source, a human intelligence source, a
        relationship with an intelligence or security service of a foreign government or international
        organization, or a non-human intelligence source; or impair the effectiveness of an intelligence method
        currently in use, available for use, or under development.
    :cvar VALUE_25_X1_EO_12951: "25X1, EO 12951" (prescribed by the DNI for use on information described in E.O.
        12951, Release of Imagery Acquired by Space-Based National Intelligence Reconnaissance Systems)
    :cvar VALUE_25_X2: Reveal information that would assist in the development, production, or use of weapons of
        mass destruction.
    :cvar VALUE_25_X3: Reveal information that would impair U.S. cryptologic systems or activities.
    :cvar VALUE_25_X4: Reveal information that would impair the application of state-of-the-art technology within
        a U.S. weapon system.
    :cvar VALUE_25_X5: Reveal formally named or numbered U.S. military war plans that remain in effect, or reveal
        operational or tactical elements of prior plans that are contained in such active plans;
    :cvar VALUE_25_X6: Reveal information, including foreign government information, that would cause serious
        harm to relations between the United States and a foreign government, or to ongoing diplomatic activities
        of the United States
    :cvar VALUE_25_X7: Reveal information that would impair the current ability of United States Government
        officials to protect the President, Vice President, and other protectees for whom protection services, in
        the interest of the national security, are authorized.
    :cvar VALUE_25_X8: Reveal information that would seriously impair current national security emergency
        preparedness plans or reveal current vulnerabilities of systems, installations, or infrastructures
        relating to the national security.
    :cvar VALUE_25_X9: Violate a statute, treaty, or international agreement that does not permit the automatic
        or unilateral declassification of information at 25 years.
    :cvar VALUE_50_X1_HUM: When the information clearly and demonstrably could be expected to reveal the identity
        of a confidential human source or a human intelligence source.
    :cvar VALUE_50_X1: The ISCAP has authorized use of this code in the FBI’s classification guidance (which
        results in a 75-year classification period) for any agency sourcing/reusing the information.
    :cvar VALUE_50_X2_WMD: When the information clearly and demonstrably could reveal key design concepts of
        weapons of mass destruction.
    :cvar VALUE_50_X6: The ISCAP has authorized use of this code in the FBI’s classification guidance (which
        results in a 75-year classification period) for any agency sourcing/reusing the information.
    """

    AEA = "AEA"
    NATO = "NATO"
    NATO_AEA = "NATO-AEA"
    VALUE_25_X1 = "25X1"
    VALUE_25_X1_EO_12951 = "25X1-EO-12951"
    VALUE_25_X2 = "25X2"
    VALUE_25_X3 = "25X3"
    VALUE_25_X4 = "25X4"
    VALUE_25_X5 = "25X5"
    VALUE_25_X6 = "25X6"
    VALUE_25_X7 = "25X7"
    VALUE_25_X8 = "25X8"
    VALUE_25_X9 = "25X9"
    VALUE_50_X1_HUM = "50X1-HUM"
    VALUE_50_X1 = "50X1"
    VALUE_50_X2_WMD = "50X2-WMD"
    VALUE_50_X6 = "50X6"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

# Telling flake8 to not flag errors in this file. It is normal that these classes are imported but not used in an
# __init__.py file.
# flake8: noqa
"""
The image_processing package contains various utilities for manipulating overhead imagery.

Image Tiling: Tiling With Updated Image Metadata
************************************************

Many applications break large remote sensing images into smaller chips or tiles for distributed processing or
dissemination. GDAL's Translate function provides basic capabilities, but it does not correctly update geospatial
metadata to reflect the new image extent. These utilities provide those functions so tile consumers can correctly
interpret the pixel information they have been provided.

.. code-block:: python
    :caption: Example showing creation of a NITF tile from the upper left corner of an image

    # Load the image and create a sensor model
    ds, sensor_model = load_gdal_dataset("./imagery/sample.nitf")
    tile_factory = GDALTileFactory(ds,
                                   sensor_model,
                                   GDALImageFormats.NITF,
                                   GDALCompressionOptions.NONE
                                   )

    # Bounds are [left_x, top_y, width, height]
    nitf_encoded_tile_bytes = tile_factory.create_encoded_tile([0, 0, 1024, 1024])


Image Tiling: Tiles for Display
*******************************

Some images, for example 11-bit panchromatic images or SAR imagery with floating point complex data, can not be
displayed directly without remapping the pixels into an 8-bit per pixel grayscale or RGB color model. The TileFactory
supports creation of tiles suitable for human review by setting both the output_type and range_adjustment options.
Note that the output_size parameter can be used to generate lower resolution tiles. This operation will make use of
GDAL generated overviews if they are available to the dataset.

.. code-block:: python
    :caption: Example showing creation of a PNG tile scaled down from the full resolution image

    viz_tile_factory = GDALTileFactory(ds,
                                       sensor_model,
                                       GDALImageFormats.PNG,
                                       GDALCompressionOptions.NONE,
                                       output_type=gdalconst.GDT_Byte,
                                       range_adjustment=RangeAdjustmentType.DRA)

    viz_tile = viz_tile_factory.create_encoded_tile([0, 0, 1024, 1024], output_size=(512, 512))

Image Tiling: Map Tiles / Orthophotos
*************************************

The TileFactory supports creation of tiles suitable for use by geographic information systems (GIS) or map-based
visualization tools. Given a north-east aligned bounding box in geographic coordinates the tile factory can use
the sensor models to orthorectify imagery to remove the perspective and terrain effects.

.. code-block:: python
    :caption: Example showing creation of a map tile from the WebMercatorQuad tile set

    # Look up the tile boundary for a tile in a well known tile set
    tile_set = MapTileSetFactory.get_for_id("WebMercatorQuad")
    tile_id = MapTileId(tile_matrix=16, tile_row=37025, tile_col=54816)
    tile = tile_set.get_tile(tile_id)

    # Create an orthophoto for this tile
    image_bytes = viz_tile_factory.create_orthophoto_tile(geo_bbox=tile.bounds, tile_size=tile.size)

.. figure:: ../images/MapTileExample-BeforeAfter.png
    :width: 600
    :alt: Original image with perspective effects and same area after orthorectification

    Example showing original image with perspective effects and same area after orthorectification.

.. figure:: ../images/MapTileExample-MapOverlay.png
    :width: 400
    :alt: Orthophoto tile overlaid on Google Maps

    Example showing map tile overlaid on Google Maps

Complex SAR Data Display
************************

There are a variety of different techniques to convert complex SAR data to a simple image suitable for human display.
The toolkit contains two helper functions that can convert complex image data into an 8-bit grayscle representation
The equations implemented are described in Sections 3.1 and 3.2 of SAR Image Scaling, Dynamic Range, Radiometric
Calibration, and Display (SAND2019-2371).

.. code-block:: python
    :caption: Example converting complex SAR data into a 8-bit per pixel image for display

    import numpy as np
    from aws.osml.image_processing import histogram_stretch, quarter_power_image

    sicd_dataset, sensor_model = load_gdal_dataset("./sample-sicd.nitf")
    complex_pixels = sicd_dataset.ReadAsArray()

    histo_stretch_pixels = histogram_stretch(complex_pixels)
    quarter_power_pixels = quarter_power_image(complex_pixels)


.. figure:: ../images/SAR-HistogramStretchImage.png
   :width: 400
   :alt: Histogram Stretch Applied to Sample SICD Image

   Example of applying histogram_stretch to a sample SICD image.


.. figure:: ../images/SAR-QuarterPowerImage.png
   :width: 400
   :alt: Quarter Power Image Applied to Sample SICD Image

   Example of applying quarter_power_image to a sample SICD image.


-------------------------

APIs
****
"""

from .gdal_tile_factory import GDALTileFactory
from .map_tileset import MapTile, MapTileId, MapTileSet
from .map_tileset_factory import MapTileSetFactory, WellKnownMapTileSet
from .sar_complex_imageop import histogram_stretch, quarter_power_image

__all__ = [
    "GDALTileFactory",
    "MapTile",
    "MapTileId",
    "MapTileSet",
    "MapTileSetFactory",
    "WellKnownMapTileSet",
    "histogram_stretch",
    "quarter_power_image",
]



#  Copyright 2024 Amazon.com, Inc. or its affiliates.

from enum import Enum
from typing import Optional

from .map_tileset import MapTileSet
from .map_tileset_wmq import WebMercatorQuadMapTileSet


class WellKnownMapTileSet(str, Enum):
    """
    A partial list of well known tile sets used by this library.
    """

    WEB_MERCATOR_QUAD = "WebMercatorQuad"
    WEB_MERCATOR_QUAD_X2 = "WebMercatorQuadx2"


class MapTileSetFactory:
    """
    This class provides a means to construct / access the implementation of well known tile sets using their
    name. It allows clients to easily work with the TileSet abstraction independent of any implementation details
    associated with specific tile sets.
    """

    @staticmethod
    def get_for_id(tile_matrix_set_id: str) -> Optional[MapTileSet]:
        """
        Constructs a tile set matching the requested id.

        :param tile_matrix_set_id: the tile set id
        :return: the TileSet or None if not available
        """
        if tile_matrix_set_id == WellKnownMapTileSet.WEB_MERCATOR_QUAD.value:
            return WebMercatorQuadMapTileSet(tile_size=256, tile_matrix_set_id=WellKnownMapTileSet.WEB_MERCATOR_QUAD.value)
        elif tile_matrix_set_id == WellKnownMapTileSet.WEB_MERCATOR_QUAD_X2.value:
            return WebMercatorQuadMapTileSet(
                tile_size=512, tile_matrix_set_id=WellKnownMapTileSet.WEB_MERCATOR_QUAD_X2.value
            )
        else:
            return None



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:ism-cvenum"


class CVEnumISMSCIControlsValuesvalue(Enum):
    """
    :cvar HCS: HCS
    :cvar KDK: Klondike
    :cvar SI: COMINT
    :cvar SI_G: SI-GAMMA
    :cvar TK: TALENT KEYHOLE
    """

    HCS = "HCS"
    KDK = "KDK"
    SI = "SI"
    SI_G = "SI-G"
    TK = "TK"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
from typing import Optional, Union

from xsdata.formats.dataclass.parsers import XmlParser

import aws.osml.formats.sidd.models.sidd_v1_0_0 as sidd100
import aws.osml.formats.sidd.models.sidd_v2_0_0 as sidd200
import aws.osml.formats.sidd.models.sidd_v3_0_0 as sidd300

from ..photogrammetry import (
    ChippedImageSensorModel,
    ImageCoordinate,
    PlaneProjectionSet,
    SARImageCoordConverter,
    SensorModel,
    SICDSensorModel,
    WorldCoordinate,
)
from .sensor_model_builder import SensorModelBuilder
from .sicd_sensor_model_builder import poly2d_to_native, xyzpoly_to_native, xyztype_to_ndarray

logger = logging.getLogger(__name__)


class SIDDSensorModelBuilder(SensorModelBuilder):
    """
    This builder is used to create sensor models for images that have SIDD metadata. The metadata is provided
    as XML that conforms to the SIDD specifications. We intend to support multiple SIDD versions but the current
    software was implemented using the v2.0.0 and v3.0.0 specifications.

    Note that the SIDD sensor models rely heavily on the SICD projections so the class of the returned model
    will be a SICDSensorModel. Future versions may rename this to SISensorModel or SARSensorModel.
    """

    def __init__(self, sidd_xml: str):
        """
        Construct the builder given the SIDD XML.

        :param sidd_xml: the XML string
        """
        super().__init__()
        self.sidd_xml = sidd_xml

    def build(self) -> Optional[SensorModel]:
        """
        Attempt to build a precise SAR sensor model. This sensor model handles chipped images natively.

        :return: the sensor model; if available
        """
        try:
            if self.sidd_xml is None or len(self.sidd_xml) == 0:
                return None

            parser = XmlParser()
            sicd = parser.from_string(self.sidd_xml)
            return SIDDSensorModelBuilder.from_dataclass(sicd)
        except Exception as e:
            logging.error("Exception caught attempting to build SIDD sensor model.", e)
        return None

    @staticmethod
    def from_dataclass(sidd: Union[sidd100.SIDD, sidd200.SIDD, sidd300.SIDD]) -> Optional[SensorModel]:
        """
        This method constructs a SIDD sensor model from the python dataclasses generated when parsing the XML. If
        the metadata shows that this is a chip then a ChippedImageSensorModel will be constructed to wrap the
        SICDSensorModel used for the full image.

        :param sidd: the dataclass object constructed from the XML
        :return: the sensor model; if available
        """

        plane_projection = sidd.measurement.plane_projection
        scp_ecf = WorldCoordinate(xyztype_to_ndarray(plane_projection.reference_point.ecef))
        scp_pixel = ImageCoordinate([plane_projection.reference_point.point.col, plane_projection.reference_point.point.row])
        time_coa_poly = poly2d_to_native(plane_projection.time_coapoly)
        arp_poly = xyzpoly_to_native(sidd.measurement.arppoly)

        u_row = xyztype_to_ndarray(plane_projection.product_plane.row_unit_vector)
        u_col = xyztype_to_ndarray(plane_projection.product_plane.col_unit_vector)
        coord_converter = SARImageCoordConverter(
            scp_pixel=scp_pixel,
            scp_ecf=scp_ecf,
            u_row=u_row,
            u_col=u_col,
            row_ss=plane_projection.sample_spacing.row,
            col_ss=plane_projection.sample_spacing.col,
            first_pixel=ImageCoordinate([0, 0]),
        )

        projection_set = PlaneProjectionSet(
            scp_ecf=scp_ecf,
            image_plane_urow=u_row,
            image_plane_ucol=u_col,
            coa_time_poly=time_coa_poly,
            arp_poly=arp_poly,
        )

        u_gpn = SICDSensorModel.compute_u_gpn(scp_ecf=scp_ecf, u_row=u_row, u_col=u_col)

        sidd_sensor_model = SICDSensorModel(
            coord_converter=coord_converter,
            coa_projection_set=projection_set,
            u_spn=u_gpn,
            u_gpn=u_gpn,
        )

        if sidd.downstream_reprocessing is None or sidd.downstream_reprocessing.geometric_chip is None:
            return sidd_sensor_model
        else:
            # Since this SIDD image is a chip of a full image wrap the regular sensor model in a sensor model that
            # will handle the conversions between the chipped image coordinates and the full image coordinates.
            # This 4 corner transformation handles images that are cropped, rotated, and scaled, from the full
            # SIDD image grid.
            geo_chip = sidd.downstream_reprocessing.geometric_chip
            chip_num_rows = geo_chip.chip_size.row
            chip_num_cols = geo_chip.chip_size.col

            chipped_image_coords = [
                ImageCoordinate(coord)
                for coord in [[0, 0], [chip_num_cols, 0], [chip_num_cols, chip_num_rows], [0, chip_num_rows]]
            ]
            full_image_coords = [
                ImageCoordinate([x.col, x.row])
                for x in [
                    geo_chip.original_upper_left_coordinate,
                    geo_chip.original_upper_right_coordinate,
                    geo_chip.original_lower_right_coordinate,
                    geo_chip.original_lower_left_coordinate,
                ]
            ]

            return ChippedImageSensorModel(
                full_image_coords,
                chipped_image_coords,
                sidd_sensor_model,
            )



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
from typing import Optional, Tuple

import numpy as np

TWO_PI = np.pi * 2.0

logger = logging.getLogger(__name__)


def image_pixels_to_complex(
    image_pixels: np.ndarray, pixel_type: Optional[str] = None, amplitude_table: Optional[np.typing.ArrayLike] = None
) -> np.ndarray:
    """
    This function converts SAR pixels from SICD imagery into complex values using equations
    found in SICD Volume 1 Section 4.2.

    :param image_pixels: the SAR image pixels
    :param pixel_type: "AMP8I_PHS8I", "RE32F_IM32F", or "RE16I_IM16I"
    :param amplitude_table: optional lookup table of amplitude values for AMP8I_PHS8I image pixels
    :return:
    """

    if pixel_type is None or pixel_type in ["RE32F_IM32F", "RE16I_IM16I"]:
        # For these pixel types the complex value is already stored in the file
        return image_pixels
    elif pixel_type == "AMP8I_PHS8I":
        # If the data is 8-bit amplitude/phase with an optional amplitude lookup table need to
        # convert it to the complex image value
        amplitude = image_pixels[0]
        phase = image_pixels[1] / 256.0
        if amplitude_table is not None:
            amplitude_lut = np.array(amplitude_table)
            amplitude = amplitude_lut[amplitude]
        return np.array([amplitude * np.cos(TWO_PI * phase), amplitude * np.sin(TWO_PI * phase)])
    else:
        raise ValueError(f"Unknown SAR Pixel Type: {pixel_type}")


def complex_to_power_value(complex_data: np.ndarray) -> np.ndarray:
    """
    This function converts SAR complex data into the pixel power values (sometimes
    called pixel intensity) using the equation found in SICD Volume 1 Section 4.10.

    :param complex_data: the SAR complex image signal with real and imaginary components
    :return: the power values
    """
    return np.sum(np.square(complex_data), axis=0)


def power_value_in_decibels(power_values: np.ndarray) -> np.ndarray:
    """
    This function converts SAR power values to decibels using the equation found in SICD Volume 1 Section 4.10.

    :param power_values: the SAR power values
    :return: the power values in decibels
    """
    return 10.0 * np.log10(power_values)


def get_value_bounds(magnitude_values: np.ndarray) -> Tuple[float, float]:
    """
    This function calculates the minimum and maximum of a set of values.

    :param magnitude_values: SAR magnitude values
    :return: (min value, max value)
    """
    return np.min(magnitude_values), np.max(magnitude_values)


def linear_mapping(magnitude_values: np.ndarray) -> np.ndarray:
    """
    This function accepts an array of magnitude values and scales them to be in the range [0:1].

    :param magnitude_values: SAR magnitude values
    :return: the scaled values in range [0:1]
    """
    min_value, max_value = get_value_bounds(magnitude_values)
    if max_value == min_value:
        return np.full(magnitude_values.shape, 0.5)

    return np.clip((magnitude_values.astype(np.float32) - min_value) / (np.float32(max_value) - min_value), 0, 1.0)


def histogram_stretch_mag_values(magnitude_values: np.ndarray, scale_factor: float = 8.0):
    """
    This function converts image pixel magnitudes to an 8-bit image by scaling the pixels and
    cropping to the desired range. This is histogram stretching without any gamma correction.

    :param magnitude_values: SAR magnitude values
    :param scale_factor: a scale factor, default = 8.0
    :return: the quantized grayscale image clipped to the range of [0:255]
    """
    mean_value = np.mean(magnitude_values[np.isfinite(magnitude_values)])
    u = 1 / (scale_factor * mean_value)
    return np.clip(255.0 * u * magnitude_values, 0.0, 255.0)


def quarter_power_mag_values(
    magnitude_values: np.ndarray, scale_factor: float = 3.0, precomputed_mean: Optional[float] = None
):
    """
    This function converts image pixel magnitudes to a Quarter-Power Image using equations
    found in Section 3.2 of SAR Image Scaling, Dynamic Range, Radiometric Calibration, and Display
    (SAND2019-2371).

    :param magnitude_values: SAR magnitude values
    :param scale_factor: a brightness factor that is typically between 5 and 3
    :param precomputed_mean: a precomputed mean of magnitude usually taken from a larger set of pixels
    :return: the quantized grayscale image clipped to the range of [0:255]
    """
    sqrt_magnitude = np.sqrt(np.abs(magnitude_values))
    mean_value = precomputed_mean
    if mean_value is None:
        mean_value = np.mean(sqrt_magnitude[np.isfinite(sqrt_magnitude)])
    b = 1 / (scale_factor * mean_value)
    return np.clip(255.0 * b * sqrt_magnitude, 0.0, 255.0)


def histogram_stretch(
    image_pixels: np.ndarray,
    pixel_type: Optional[str] = None,
    amplitude_table: Optional[np.typing.ArrayLike] = None,
    scale_factor: float = 8.0,
) -> np.ndarray:
    """
    This function converts SAR image pixels to an 8-bit grayscale image by scaling the pixels and
    cropping to the desired range [0:255]. This is histogram stretching without any gamma correction.
    The equations are described in Section 3.1 of SAR Image Scaling, Dynamic Range, Radiometric Calibration,
    and Display (SAND2019-2371).

    :param image_pixels: the SAR image pixels
    :param pixel_type: "AMP8I_PHS8I", "RE32F_IM32F", or "RE16I_IM16I"
    :param amplitude_table: optional lookup table of amplitude values for AMP8I_PHS8I image pixels
    :param scale_factor: a scale factor, default = 8.0
    :return: the quantized grayscale image clipped to the range of [0:255]
    """
    complex_data = image_pixels_to_complex(image_pixels, pixel_type=pixel_type, amplitude_table=amplitude_table)
    power_values = complex_to_power_value(complex_data)
    return histogram_stretch_mag_values(power_values, scale_factor=scale_factor)


def quarter_power_image(
    image_pixels: np.ndarray,
    pixel_type: Optional[str] = None,
    amplitude_table: Optional[np.typing.ArrayLike] = None,
    scale_factor: float = 3.0,
    precomputed_mean: Optional[float] = None,
) -> np.ndarray:
    """
    This function converts SAR image pixels to an 8-bit grayscale image pixel magnitudes to a Quarter-Power
    Image using equations found in Section 3.2 of SAR Image Scaling, Dynamic Range, Radiometric Calibration,
    and Display (SAND2019-2371).

    :param image_pixels: the SAR image pixels
    :param pixel_type: "AMP8I_PHS8I", "RE32F_IM32F", or "RE16I_IM16I"
    :param amplitude_table: optional lookup table of amplitude values for AMP8I_PHS8I image pixels
    :param scale_factor: a brightness factor that is typically between 5 and 3
    :param precomputed_mean: a precomputed mean of magnitude usually taken from a larger set of pixels
    :return: the quantized grayscale image clipped to the range of [0:255]
    """
    complex_data = image_pixels_to_complex(image_pixels, pixel_type=pixel_type, amplitude_table=amplitude_table)
    power_values = complex_to_power_value(complex_data)
    return quarter_power_mag_values(power_values, scale_factor=scale_factor, precomputed_mean=precomputed_mean)



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from .sfa import (
    AbstractReferenceSystemType,
    CurveType,
    DatumType,
    GeocentricCoordinateSystemType,
    GeographicCoordinateSystemType,
    GeometryCollectionType,
    GeometryType,
    LinearRingType,
    LineStringType,
)
from .sfa import LineType as SfaLineType
from .sfa import MultiCurveType, MultiLineStringType, MultiPointType, MultiPolygonType, MultiSurfaceType
from .sfa import ParameterType as SfaParameterType
from .sfa import PointType
from .sfa import PolygonType as SfaPolygonType
from .sfa import (
    PolyhedralSurfaceType,
    PrimeMeridianType,
    ProjectedCoordinateSystemType,
    ProjectionType,
    ReferenceSystemType,
    SpheriodType,
    SurfaceType,
    TriangleType,
    TriangulatedIrregularNetworkType,
    UNITType,
)
from .sicommon_types import AngleMagnitudeType as AngleMagnitudeType
from .sicommon_types import ArrayDoubleType as ArrayDoubleType
from .sicommon_types import CollectIdentifierType as CollectIdentifierType
from .sicommon_types import CollectionInfoType as CollectionInfoType
from .sicommon_types import ComplexType as ComplexType
from .sicommon_types import CornerStringType as CornerStringType
from .sicommon_types import ErrorFrameType as ErrorFrameType
from .sicommon_types import ErrorStatisticsType as ErrorStatisticsType
from .sicommon_types import GammaZeroSFIncidenceMapType
from .sicommon_types import ImageCreationType as ImageCreationType
from .sicommon_types import LatLonCornerStringType as LatLonCornerStringType
from .sicommon_types import LatLonCornerType as LatLonCornerType
from .sicommon_types import LatLonHAECornerType as LatLonHAECornerType
from .sicommon_types import LatLonType as LatLonType
from .sicommon_types import LatLonVertexType as LatLonVertexType
from .sicommon_types import LineType as SicommontypesLineType
from .sicommon_types import LLHCornerStringType as LLHCornerStringType
from .sicommon_types import LLHType as LLHType
from .sicommon_types import ModeIdentifierType as ModeIdentifierType
from .sicommon_types import ParameterType as SicommontypesParameterType
from .sicommon_types import PolarizationType
from .sicommon_types import Poly1DType as Poly1DType
from .sicommon_types import Poly2DType as Poly2DType
from .sicommon_types import PolyCoef1DType as PolyCoef1DType
from .sicommon_types import PolyCoef2DType as PolyCoef2DType
from .sicommon_types import PolygonType as SicommontypesPolygonType
from .sicommon_types import RadarModeType as RadarModeType
from .sicommon_types import RadiometricType as RadiometricType
from .sicommon_types import RangeAzimuthType as RangeAzimuthType
from .sicommon_types import ReferencePointType as ReferencePointType
from .sicommon_types import RowColDoubleType as RowColDoubleType
from .sicommon_types import RowColIntType as RowColIntType
from .sicommon_types import RowColVertexType as RowColVertexType
from .sicommon_types import SigmaZeroSFIncidenceMapType
from .sicommon_types import ValidDataType as ValidDataType
from .sicommon_types import XYZAttributeType as XYZAttributeType
from .sicommon_types import XYZPolyAttributeType as XYZPolyAttributeType
from .sicommon_types import XYZPolyType as XYZPolyType
from .sicommon_types import XYZType as XYZType
from .sicommon_types_v1_0 import AngleMagnitudeType as V10AngleMagnitudeType
from .sicommon_types_v1_0 import AngleZeroToExclusive360MagnitudeType
from .sicommon_types_v1_0 import ArrayDoubleType as V10ArrayDoubleType
from .sicommon_types_v1_0 import CollectIdentifierType as V10CollectIdentifierType
from .sicommon_types_v1_0 import CollectionInfoType as V10CollectionInfoType
from .sicommon_types_v1_0 import ComplexType as V10ComplexType
from .sicommon_types_v1_0 import CornerStringType as V10CornerStringType
from .sicommon_types_v1_0 import ErrorFrameType as V10ErrorFrameType
from .sicommon_types_v1_0 import ErrorStatisticsType as V10ErrorStatisticsType
from .sicommon_types_v1_0 import GeoInfo, GeoInfoType
from .sicommon_types_v1_0 import ImageCreationType as V10ImageCreationType
from .sicommon_types_v1_0 import LatLonCornerStringType as V10LatLonCornerStringType
from .sicommon_types_v1_0 import LatLonCornerType as V10LatLonCornerType
from .sicommon_types_v1_0 import LatLonHAECornerType as V10LatLonHAECornerType
from .sicommon_types_v1_0 import LatLonRestrictionType
from .sicommon_types_v1_0 import LatLonType as V10LatLonType
from .sicommon_types_v1_0 import LatLonVertexType as V10LatLonVertexType
from .sicommon_types_v1_0 import LineType as V10LineType
from .sicommon_types_v1_0 import LLHCornerStringType as V10LLHCornerStringType
from .sicommon_types_v1_0 import LLHType as V10LLHType
from .sicommon_types_v1_0 import MatchInfoType
from .sicommon_types_v1_0 import ModeIdentifierType as V10ModeIdentifierType
from .sicommon_types_v1_0 import NoiseLevelNoiseLevelType
from .sicommon_types_v1_0 import ParameterType as V10ParameterType
from .sicommon_types_v1_0 import Polarization1Typevalue
from .sicommon_types_v1_0 import Poly1DType as V10Poly1DType
from .sicommon_types_v1_0 import Poly2DType as V10Poly2DType
from .sicommon_types_v1_0 import PolyCoef1DType as V10PolyCoef1DType
from .sicommon_types_v1_0 import PolyCoef2DType as V10PolyCoef2DType
from .sicommon_types_v1_0 import PolygonType as V10PolygonType
from .sicommon_types_v1_0 import RadarModeType as V10RadarModeType
from .sicommon_types_v1_0 import RadiometricType as V10RadiometricType
from .sicommon_types_v1_0 import RadiometricTypeSigmaZeroSFIncidenceMap
from .sicommon_types_v1_0 import RangeAzimuthType as V10RangeAzimuthType
from .sicommon_types_v1_0 import ReferencePointType as V10ReferencePointType
from .sicommon_types_v1_0 import RowColDoubleType as V10RowColDoubleType
from .sicommon_types_v1_0 import RowColIntType as V10RowColIntType
from .sicommon_types_v1_0 import RowColVertexType as V10RowColVertexType
from .sicommon_types_v1_0 import ValidDataType as V10ValidDataType
from .sicommon_types_v1_0 import XYZAttributeType as V10XYZAttributeType
from .sicommon_types_v1_0 import XYZPolyAttributeType as V10XYZPolyAttributeType
from .sicommon_types_v1_0 import XYZPolyType as V10XYZPolyType
from .sicommon_types_v1_0 import XYZType as V10XYZType
from .sidd_v1_0_0 import SIDD as V1SIDD
from .sidd_v1_0_0 import AcheivedResolutionType as V1AcheivedResolutionType
from .sidd_v1_0_0 import AnnotationObjectType as V1AnnotationObjectType
from .sidd_v1_0_0 import AnnotationsType as V1AnnotationsType
from .sidd_v1_0_0 import AnnotationType as V1AnnotationType
from .sidd_v1_0_0 import BaseProjectionType as V1BaseProjectionType
from .sidd_v1_0_0 import ClassificationGuidanceType as V1ClassificationGuidanceType
from .sidd_v1_0_0 import ColorDisplayRemapType as V1ColorDisplayRemapType
from .sidd_v1_0_0 import CylindricalProjectionType as V1CylindricalProjectionType
from .sidd_v1_0_0 import DecimationMethodType
from .sidd_v1_0_0 import DownstreamReprocessingType as V1DownstreamReprocessingType
from .sidd_v1_0_0 import DRAHistogramOverridesType as V1DRAHistogramOverridesType
from .sidd_v1_0_0 import ExploitationFeaturesCollectionGeometryType as V1ExploitationFeaturesCollectionGeometryType
from .sidd_v1_0_0 import ExploitationFeaturesCollectionInformationType as V1ExploitationFeaturesCollectionInformationType
from .sidd_v1_0_0 import ExploitationFeaturesCollectionPhenomenologyType as V1ExploitationFeaturesCollectionPhenomenologyType
from .sidd_v1_0_0 import ExploitationFeaturesCollectionType as V1ExploitationFeaturesCollectionType
from .sidd_v1_0_0 import ExploitationFeaturesProductType as V1ExploitationFeaturesProductType
from .sidd_v1_0_0 import ExploitationFeaturesType as V1ExploitationFeaturesType
from .sidd_v1_0_0 import FootprintType, GeographicAndTargetType, GeographicCoverageType, GeographicInformationType
from .sidd_v1_0_0 import GeographicProjectionType as V1GeographicProjectionType
from .sidd_v1_0_0 import GeometricChipType as V1GeometricChipType
from .sidd_v1_0_0 import InputROIType as V1InputROIType
from .sidd_v1_0_0 import Lookup3TableType as V1Lookup3TableType
from .sidd_v1_0_0 import LookupTableType as V1LookupTableType
from .sidd_v1_0_0 import MagnificationMethodType
from .sidd_v1_0_0 import MeasurableProjectionType as V1MeasurableProjectionType
from .sidd_v1_0_0 import MeasurementType as V1MeasurementType
from .sidd_v1_0_0 import MonitorCompensationAppliedType
from .sidd_v1_0_0 import MonochromeDisplayRemapType as V1MonochromeDisplayRemapType
from .sidd_v1_0_0 import PixelType as V1PixelType
from .sidd_v1_0_0 import PlaneProjectionType as V1PlaneProjectionType
from .sidd_v1_0_0 import PolynomialProjectionType as V1PolynomialProjectionType
from .sidd_v1_0_0 import ProcessingEventType as V1ProcessingEventType
from .sidd_v1_0_0 import ProcessingModuleType as V1ProcessingModuleType
from .sidd_v1_0_0 import ProcessorInformationType as V1ProcessorInformationType
from .sidd_v1_0_0 import ProductClassificationType as V1ProductClassificationType
from .sidd_v1_0_0 import ProductCreationType as V1ProductCreationType
from .sidd_v1_0_0 import ProductDisplayType as V1ProductDisplayType
from .sidd_v1_0_0 import ProductPlaneType as V1ProductPlaneType
from .sidd_v1_0_0 import ProductProcessingType as V1ProductProcessingType
from .sidd_v1_0_0 import RemapChoiceType as V1RemapChoiceType
from .sidd_v1_0_0 import TargetInformationType
from .sidd_v1_0_0 import TxRcvPolarizationType as V1TxRcvPolarizationType
from .sidd_v2_0_0 import SIDD as V2SIDD
from .sidd_v2_0_0 import AccuracyType as V2AccuracyType
from .sidd_v2_0_0 import AcheivedResolutionType as V2AcheivedResolutionType
from .sidd_v2_0_0 import AnnotationObjectType as V2AnnotationObjectType
from .sidd_v2_0_0 import AnnotationsType as V2AnnotationsType
from .sidd_v2_0_0 import AnnotationType as V2AnnotationType
from .sidd_v2_0_0 import BandEqualizationType as V2BandEqualizationType
from .sidd_v2_0_0 import BankCustomType as V2BankCustomType
from .sidd_v2_0_0 import BaseProjectionType as V2BaseProjectionType
from .sidd_v2_0_0 import ClassificationGuidanceType as V2ClassificationGuidanceType
from .sidd_v2_0_0 import ColorDisplayRemapType as V2ColorDisplayRemapType
from .sidd_v2_0_0 import ColorManagementModuleType as V2ColorManagementModuleType
from .sidd_v2_0_0 import ColorSpaceTransformType as V2ColorSpaceTransformType
from .sidd_v2_0_0 import CompressionType as V2CompressionType
from .sidd_v2_0_0 import CustomLookupType as V2CustomLookupType
from .sidd_v2_0_0 import CylindricalProjectionType as V2CylindricalProjectionType
from .sidd_v2_0_0 import DigitalElevationDataType as V2DigitalElevationDataType
from .sidd_v2_0_0 import DownsamplingMethodType as V2DownsamplingMethodType
from .sidd_v2_0_0 import DownstreamReprocessingType as V2DownstreamReprocessingType
from .sidd_v2_0_0 import DRAHistogramOverridesType as V2DRAHistogramOverridesType
from .sidd_v2_0_0 import DRAOverrides as V2DRAOverrides
from .sidd_v2_0_0 import DRAParameters as V2DRAParameters
from .sidd_v2_0_0 import DynamicRangeAdjustmentType as V2DynamicRangeAdjustmentType
from .sidd_v2_0_0 import EarthModelType as V2EarthModelType
from .sidd_v2_0_0 import EqualizationAlgorithmType as V2EqualizationAlgorithmType
from .sidd_v2_0_0 import ExploitationFeaturesCollectionGeometryType as V2ExploitationFeaturesCollectionGeometryType
from .sidd_v2_0_0 import ExploitationFeaturesCollectionInformationType as V2ExploitationFeaturesCollectionInformationType
from .sidd_v2_0_0 import ExploitationFeaturesCollectionPhenomenologyType as V2ExploitationFeaturesCollectionPhenomenologyType
from .sidd_v2_0_0 import ExploitationFeaturesCollectionType as V2ExploitationFeaturesCollectionType
from .sidd_v2_0_0 import ExploitationFeaturesProductType as V2ExploitationFeaturesProductType
from .sidd_v2_0_0 import ExploitationFeaturesType as V2ExploitationFeaturesType
from .sidd_v2_0_0 import FilterBankCoefType as V2FilterBankCoefType
from .sidd_v2_0_0 import FilterBankType as V2FilterBankType
from .sidd_v2_0_0 import FilterDatabaseNameType as V2FilterDatabaseNameType
from .sidd_v2_0_0 import FilterKernelCoefType as V2FilterKernelCoefType
from .sidd_v2_0_0 import FilterKernelType as V2FilterKernelType
from .sidd_v2_0_0 import FilterOperationType as V2FilterOperationType
from .sidd_v2_0_0 import FilterType as V2FilterType
from .sidd_v2_0_0 import GeoDataType as V2GeoDataType
from .sidd_v2_0_0 import GeographicCoordinatesType as V2GeographicCoordinatesType
from .sidd_v2_0_0 import GeographicProjectionType as V2GeographicProjectionType
from .sidd_v2_0_0 import GeometricChipType as V2GeometricChipType
from .sidd_v2_0_0 import GeometricTransformType as V2GeometricTransformType
from .sidd_v2_0_0 import GeopositioningType as V2GeopositioningType
from .sidd_v2_0_0 import GeopositioningTypeCoordinateSystemType as V2GeopositioningTypeCoordinateSystemType
from .sidd_v2_0_0 import GeopositioningTypeGeodeticDatum as V2GeopositioningTypeGeodeticDatum
from .sidd_v2_0_0 import GeopositioningTypeReferenceEllipsoid as V2GeopositioningTypeReferenceEllipsoid
from .sidd_v2_0_0 import GeopositioningTypeSoundingDatum as V2GeopositioningTypeSoundingDatum
from .sidd_v2_0_0 import GeopositioningTypeVerticalDatum as V2GeopositioningTypeVerticalDatum
from .sidd_v2_0_0 import ImageCornersType as V2ImageCornersType
from .sidd_v2_0_0 import InputROIType as V2InputROIType
from .sidd_v2_0_0 import InteractiveProcessingType as V2InteractiveProcessingType
from .sidd_v2_0_0 import J2KSubtype as V2J2KSubtype
from .sidd_v2_0_0 import J2KType as V2J2KType
from .sidd_v2_0_0 import KernelCustomType as V2KernelCustomType
from .sidd_v2_0_0 import LayerInfoType as V2LayerInfoType
from .sidd_v2_0_0 import LayerType as V2LayerType
from .sidd_v2_0_0 import Lookup3TableType as V2Lookup3TableType
from .sidd_v2_0_0 import LookupTableType as V2LookupTableType
from .sidd_v2_0_0 import LUTInfoType as V2LUTInfoType
from .sidd_v2_0_0 import MeasurableProjectionType as V2MeasurableProjectionType
from .sidd_v2_0_0 import MeasurementType as V2MeasurementType
from .sidd_v2_0_0 import MeasurementTypeARPFlag as V2MeasurementTypeARPFlag
from .sidd_v2_0_0 import MonochromeDisplayRemapType as V2MonochromeDisplayRemapType
from .sidd_v2_0_0 import NewLookupTableType as V2NewLookupTableType
from .sidd_v2_0_0 import NonInteractiveProcessingType as V2NonInteractiveProcessingType
from .sidd_v2_0_0 import Orientation as V2Orientation
from .sidd_v2_0_0 import PixelType as V2PixelType
from .sidd_v2_0_0 import PlaneProjectionType as V2PlaneProjectionType
from .sidd_v2_0_0 import PolygonType as V2PolygonType
from .sidd_v2_0_0 import PolynomialProjectionType as V2PolynomialProjectionType
from .sidd_v2_0_0 import PositionalAccuracyType as V2PositionalAccuracyType
from .sidd_v2_0_0 import PredefinedFilterType as V2PredefinedFilterType
from .sidd_v2_0_0 import PredefinedLookupType as V2PredefinedLookupType
from .sidd_v2_0_0 import ProcessingEventType as V2ProcessingEventType
from .sidd_v2_0_0 import ProcessingModuleType as V2ProcessingModuleType
from .sidd_v2_0_0 import ProcessorInformationType as V2ProcessorInformationType
from .sidd_v2_0_0 import ProcTxRcvPolarizationType as V2ProcTxRcvPolarizationType
from .sidd_v2_0_0 import ProductClassificationType as V2ProductClassificationType
from .sidd_v2_0_0 import ProductCreationType as V2ProductCreationType
from .sidd_v2_0_0 import ProductDisplayType as V2ProductDisplayType
from .sidd_v2_0_0 import ProductGenerationOptionsType as V2ProductGenerationOptionsType
from .sidd_v2_0_0 import ProductPlaneType as V2ProductPlaneType
from .sidd_v2_0_0 import ProductProcessingType as V2ProductProcessingType
from .sidd_v2_0_0 import RangeAdjustmentAlgorithmType as V2RangeAdjustmentAlgorithmType
from .sidd_v2_0_0 import RemapChoiceType as V2RemapChoiceType
from .sidd_v2_0_0 import RenderingIntentType as V2RenderingIntentType
from .sidd_v2_0_0 import RRDSType as V2RRDSType
from .sidd_v2_0_0 import ScalingType as V2ScalingType
from .sidd_v2_0_0 import ShadowDirectionType as V2ShadowDirectionType
from .sidd_v2_0_0 import SharpnessEnhancementType as V2SharpnessEnhancementType
from .sidd_v2_0_0 import TxRcvPolarizationType as V2TxRcvPolarizationType
from .sidd_v2_0_0 import ValidDataType as V2ValidDataType
from .sidd_v3_0_0 import SIDD as V3SIDD
from .sidd_v3_0_0 import AccuracyType as V3AccuracyType
from .sidd_v3_0_0 import AcheivedResolutionType as V3AcheivedResolutionType
from .sidd_v3_0_0 import AnnotationObjectType as V3AnnotationObjectType
from .sidd_v3_0_0 import AnnotationsType as V3AnnotationsType
from .sidd_v3_0_0 import AnnotationType as V3AnnotationType
from .sidd_v3_0_0 import BandEqualizationType as V3BandEqualizationType
from .sidd_v3_0_0 import BankCustomType as V3BankCustomType
from .sidd_v3_0_0 import BaseProjectionType as V3BaseProjectionType
from .sidd_v3_0_0 import ClassificationGuidanceType as V3ClassificationGuidanceType
from .sidd_v3_0_0 import ColorDisplayRemapType as V3ColorDisplayRemapType
from .sidd_v3_0_0 import ColorManagementModuleType as V3ColorManagementModuleType
from .sidd_v3_0_0 import ColorSpaceTransformType as V3ColorSpaceTransformType
from .sidd_v3_0_0 import CompressionType as V3CompressionType
from .sidd_v3_0_0 import CustomLookupType as V3CustomLookupType
from .sidd_v3_0_0 import CylindricalProjectionType as V3CylindricalProjectionType
from .sidd_v3_0_0 import DigitalElevationDataType as V3DigitalElevationDataType
from .sidd_v3_0_0 import DownsamplingMethodType as V3DownsamplingMethodType
from .sidd_v3_0_0 import DownstreamReprocessingType as V3DownstreamReprocessingType
from .sidd_v3_0_0 import DRAHistogramOverridesType as V3DRAHistogramOverridesType
from .sidd_v3_0_0 import DRAOverrides as V3DRAOverrides
from .sidd_v3_0_0 import DRAParameters as V3DRAParameters
from .sidd_v3_0_0 import DynamicRangeAdjustmentType as V3DynamicRangeAdjustmentType
from .sidd_v3_0_0 import EarthModelType as V3EarthModelType
from .sidd_v3_0_0 import EqualizationAlgorithmType as V3EqualizationAlgorithmType
from .sidd_v3_0_0 import ExploitationFeaturesCollectionGeometryType as V3ExploitationFeaturesCollectionGeometryType
from .sidd_v3_0_0 import ExploitationFeaturesCollectionInformationType as V3ExploitationFeaturesCollectionInformationType
from .sidd_v3_0_0 import ExploitationFeaturesCollectionPhenomenologyType as V3ExploitationFeaturesCollectionPhenomenologyType
from .sidd_v3_0_0 import ExploitationFeaturesCollectionType as V3ExploitationFeaturesCollectionType
from .sidd_v3_0_0 import ExploitationFeaturesProductType as V3ExploitationFeaturesProductType
from .sidd_v3_0_0 import ExploitationFeaturesType as V3ExploitationFeaturesType
from .sidd_v3_0_0 import FilterBankCoefType as V3FilterBankCoefType
from .sidd_v3_0_0 import FilterBankType as V3FilterBankType
from .sidd_v3_0_0 import FilterDatabaseNameType as V3FilterDatabaseNameType
from .sidd_v3_0_0 import FilterKernelCoefType as V3FilterKernelCoefType
from .sidd_v3_0_0 import FilterKernelType as V3FilterKernelType
from .sidd_v3_0_0 import FilterOperationType as V3FilterOperationType
from .sidd_v3_0_0 import FilterType as V3FilterType
from .sidd_v3_0_0 import GeoDataType as V3GeoDataType
from .sidd_v3_0_0 import GeographicCoordinatesType as V3GeographicCoordinatesType
from .sidd_v3_0_0 import GeographicProjectionType as V3GeographicProjectionType
from .sidd_v3_0_0 import GeometricChipType as V3GeometricChipType
from .sidd_v3_0_0 import GeometricTransformType as V3GeometricTransformType
from .sidd_v3_0_0 import GeopositioningType as V3GeopositioningType
from .sidd_v3_0_0 import GeopositioningTypeCoordinateSystemType as V3GeopositioningTypeCoordinateSystemType
from .sidd_v3_0_0 import GeopositioningTypeGeodeticDatum as V3GeopositioningTypeGeodeticDatum
from .sidd_v3_0_0 import GeopositioningTypeReferenceEllipsoid as V3GeopositioningTypeReferenceEllipsoid
from .sidd_v3_0_0 import GeopositioningTypeSoundingDatum as V3GeopositioningTypeSoundingDatum
from .sidd_v3_0_0 import GeopositioningTypeVerticalDatum as V3GeopositioningTypeVerticalDatum
from .sidd_v3_0_0 import ImageCornersType as V3ImageCornersType
from .sidd_v3_0_0 import InputROIType as V3InputROIType
from .sidd_v3_0_0 import InteractiveProcessingType as V3InteractiveProcessingType
from .sidd_v3_0_0 import J2KSubtype as V3J2KSubtype
from .sidd_v3_0_0 import J2KType as V3J2KType
from .sidd_v3_0_0 import KernelCustomType as V3KernelCustomType
from .sidd_v3_0_0 import LayerInfoType as V3LayerInfoType
from .sidd_v3_0_0 import LayerType as V3LayerType
from .sidd_v3_0_0 import Lookup3TableType as V3Lookup3TableType
from .sidd_v3_0_0 import LookupTableType as V3LookupTableType
from .sidd_v3_0_0 import LUTInfoType as V3LUTInfoType
from .sidd_v3_0_0 import MeasurableProjectionType as V3MeasurableProjectionType
from .sidd_v3_0_0 import MeasurementType as V3MeasurementType
from .sidd_v3_0_0 import MeasurementTypeARPFlag as V3MeasurementTypeARPFlag
from .sidd_v3_0_0 import MonochromeDisplayRemapType as V3MonochromeDisplayRemapType
from .sidd_v3_0_0 import NewLookupTableType as V3NewLookupTableType
from .sidd_v3_0_0 import NonInteractiveProcessingType as V3NonInteractiveProcessingType
from .sidd_v3_0_0 import Orientation as V3Orientation
from .sidd_v3_0_0 import PixelType as V3PixelType
from .sidd_v3_0_0 import PlaneProjectionType as V3PlaneProjectionType
from .sidd_v3_0_0 import PolygonType as V3PolygonType
from .sidd_v3_0_0 import PolynomialProjectionType as V3PolynomialProjectionType
from .sidd_v3_0_0 import PositionalAccuracyType as V3PositionalAccuracyType
from .sidd_v3_0_0 import PredefinedFilterType as V3PredefinedFilterType
from .sidd_v3_0_0 import PredefinedLookupType as V3PredefinedLookupType
from .sidd_v3_0_0 import ProcessingEventType as V3ProcessingEventType
from .sidd_v3_0_0 import ProcessingModuleType as V3ProcessingModuleType
from .sidd_v3_0_0 import ProcessorInformationType as V3ProcessorInformationType
from .sidd_v3_0_0 import ProcTxRcvPolarizationType as V3ProcTxRcvPolarizationType
from .sidd_v3_0_0 import ProductClassificationType as V3ProductClassificationType
from .sidd_v3_0_0 import ProductCreationType as V3ProductCreationType
from .sidd_v3_0_0 import ProductDisplayType as V3ProductDisplayType
from .sidd_v3_0_0 import ProductGenerationOptionsType as V3ProductGenerationOptionsType
from .sidd_v3_0_0 import ProductPlaneType as V3ProductPlaneType
from .sidd_v3_0_0 import ProductProcessingType as V3ProductProcessingType
from .sidd_v3_0_0 import RangeAdjustmentAlgorithmType as V3RangeAdjustmentAlgorithmType
from .sidd_v3_0_0 import RemapChoiceType as V3RemapChoiceType
from .sidd_v3_0_0 import RenderingIntentType as V3RenderingIntentType
from .sidd_v3_0_0 import RRDSType as V3RRDSType
from .sidd_v3_0_0 import ScalingType as V3ScalingType
from .sidd_v3_0_0 import ShadowDirectionType as V3ShadowDirectionType
from .sidd_v3_0_0 import SharpnessEnhancementType as V3SharpnessEnhancementType
from .sidd_v3_0_0 import TxRcvPolarizationType as V3TxRcvPolarizationType
from .sidd_v3_0_0 import ValidDataType as V3ValidDataType

__all__ = [
    "AbstractReferenceSystemType",
    "CurveType",
    "DatumType",
    "GeocentricCoordinateSystemType",
    "GeographicCoordinateSystemType",
    "GeometryCollectionType",
    "GeometryType",
    "LineStringType",
    "SfaLineType",
    "LinearRingType",
    "MultiCurveType",
    "MultiLineStringType",
    "MultiPointType",
    "MultiPolygonType",
    "MultiSurfaceType",
    "SfaParameterType",
    "PointType",
    "SfaPolygonType",
    "PolyhedralSurfaceType",
    "PrimeMeridianType",
    "ProjectedCoordinateSystemType",
    "ProjectionType",
    "ReferenceSystemType",
    "SpheriodType",
    "SurfaceType",
    "TriangleType",
    "TriangulatedIrregularNetworkType",
    "UNITType",
    "AngleMagnitudeType",
    "ArrayDoubleType",
    "CollectIdentifierType",
    "CollectionInfoType",
    "ComplexType",
    "CornerStringType",
    "ErrorFrameType",
    "ErrorStatisticsType",
    "GammaZeroSFIncidenceMapType",
    "ImageCreationType",
    "LLHCornerStringType",
    "LLHType",
    "LatLonCornerStringType",
    "LatLonCornerType",
    "LatLonHAECornerType",
    "LatLonType",
    "LatLonVertexType",
    "SicommontypesLineType",
    "ModeIdentifierType",
    "SicommontypesParameterType",
    "PolarizationType",
    "Poly1DType",
    "Poly2DType",
    "PolyCoef1DType",
    "PolyCoef2DType",
    "SicommontypesPolygonType",
    "RadarModeType",
    "RadiometricType",
    "RangeAzimuthType",
    "ReferencePointType",
    "RowColDoubleType",
    "RowColIntType",
    "RowColVertexType",
    "SigmaZeroSFIncidenceMapType",
    "ValidDataType",
    "XYZAttributeType",
    "XYZPolyAttributeType",
    "XYZPolyType",
    "XYZType",
    "V10AngleMagnitudeType",
    "AngleZeroToExclusive360MagnitudeType",
    "V10ArrayDoubleType",
    "V10CollectIdentifierType",
    "V10CollectionInfoType",
    "V10ComplexType",
    "V10CornerStringType",
    "V10ErrorFrameType",
    "V10ErrorStatisticsType",
    "GeoInfo",
    "GeoInfoType",
    "V10ImageCreationType",
    "V10LLHCornerStringType",
    "V10LLHType",
    "V10LatLonCornerStringType",
    "V10LatLonCornerType",
    "V10LatLonHAECornerType",
    "LatLonRestrictionType",
    "V10LatLonType",
    "V10LatLonVertexType",
    "V10LineType",
    "MatchInfoType",
    "V10ModeIdentifierType",
    "NoiseLevelNoiseLevelType",
    "V10ParameterType",
    "Polarization1Typevalue",
    "V10Poly1DType",
    "V10Poly2DType",
    "V10PolyCoef1DType",
    "V10PolyCoef2DType",
    "V10PolygonType",
    "V10RadarModeType",
    "V10RadiometricType",
    "RadiometricTypeSigmaZeroSFIncidenceMap",
    "V10RangeAzimuthType",
    "V10ReferencePointType",
    "V10RowColDoubleType",
    "V10RowColIntType",
    "V10RowColVertexType",
    "V10ValidDataType",
    "V10XYZAttributeType",
    "V10XYZPolyAttributeType",
    "V10XYZPolyType",
    "V10XYZType",
    "V1AcheivedResolutionType",
    "V1AnnotationObjectType",
    "V1AnnotationType",
    "V1AnnotationsType",
    "V1BaseProjectionType",
    "V1ClassificationGuidanceType",
    "V1ColorDisplayRemapType",
    "V1CylindricalProjectionType",
    "V1DRAHistogramOverridesType",
    "DecimationMethodType",
    "V1DownstreamReprocessingType",
    "V1ExploitationFeaturesCollectionGeometryType",
    "V1ExploitationFeaturesCollectionInformationType",
    "V1ExploitationFeaturesCollectionPhenomenologyType",
    "V1ExploitationFeaturesCollectionType",
    "V1ExploitationFeaturesProductType",
    "V1ExploitationFeaturesType",
    "FootprintType",
    "GeographicAndTargetType",
    "GeographicCoverageType",
    "GeographicInformationType",
    "V1GeographicProjectionType",
    "V1GeometricChipType",
    "V1InputROIType",
    "V1Lookup3TableType",
    "V1LookupTableType",
    "MagnificationMethodType",
    "V1MeasurableProjectionType",
    "V1MeasurementType",
    "MonitorCompensationAppliedType",
    "V1MonochromeDisplayRemapType",
    "V1PixelType",
    "V1PlaneProjectionType",
    "V1PolynomialProjectionType",
    "V1ProcessingEventType",
    "V1ProcessingModuleType",
    "V1ProcessorInformationType",
    "V1ProductClassificationType",
    "V1ProductCreationType",
    "V1ProductDisplayType",
    "V1ProductPlaneType",
    "V1ProductProcessingType",
    "V1RemapChoiceType",
    "V1SIDD",
    "TargetInformationType",
    "V1TxRcvPolarizationType",
    "V2AccuracyType",
    "V2AcheivedResolutionType",
    "V2AnnotationObjectType",
    "V2AnnotationType",
    "V2AnnotationsType",
    "V2BandEqualizationType",
    "V2BankCustomType",
    "V2BaseProjectionType",
    "V2ClassificationGuidanceType",
    "V2ColorDisplayRemapType",
    "V2ColorManagementModuleType",
    "V2ColorSpaceTransformType",
    "V2CompressionType",
    "V2CustomLookupType",
    "V2CylindricalProjectionType",
    "V2DRAHistogramOverridesType",
    "V2DRAOverrides",
    "V2DRAParameters",
    "V2DigitalElevationDataType",
    "V2DownsamplingMethodType",
    "V2DownstreamReprocessingType",
    "V2DynamicRangeAdjustmentType",
    "V2EarthModelType",
    "V2EqualizationAlgorithmType",
    "V2ExploitationFeaturesCollectionGeometryType",
    "V2ExploitationFeaturesCollectionInformationType",
    "V2ExploitationFeaturesCollectionPhenomenologyType",
    "V2ExploitationFeaturesCollectionType",
    "V2ExploitationFeaturesProductType",
    "V2ExploitationFeaturesType",
    "V2FilterBankCoefType",
    "V2FilterBankType",
    "V2FilterDatabaseNameType",
    "V2FilterKernelCoefType",
    "V2FilterKernelType",
    "V2FilterOperationType",
    "V2FilterType",
    "V2GeoDataType",
    "V2GeographicCoordinatesType",
    "V2GeographicProjectionType",
    "V2GeometricChipType",
    "V2GeometricTransformType",
    "V2GeopositioningType",
    "V2GeopositioningTypeCoordinateSystemType",
    "V2GeopositioningTypeGeodeticDatum",
    "V2GeopositioningTypeReferenceEllipsoid",
    "V2GeopositioningTypeSoundingDatum",
    "V2GeopositioningTypeVerticalDatum",
    "V2ImageCornersType",
    "V2InputROIType",
    "V2InteractiveProcessingType",
    "V2J2KSubtype",
    "V2J2KType",
    "V2KernelCustomType",
    "V2LUTInfoType",
    "V2LayerInfoType",
    "V2LayerType",
    "V2Lookup3TableType",
    "V2LookupTableType",
    "V2MeasurableProjectionType",
    "V2MeasurementType",
    "V2MeasurementTypeARPFlag",
    "V2MonochromeDisplayRemapType",
    "V2NewLookupTableType",
    "V2NonInteractiveProcessingType",
    "V2Orientation",
    "V2PixelType",
    "V2PlaneProjectionType",
    "V2PolygonType",
    "V2PolynomialProjectionType",
    "V2PositionalAccuracyType",
    "V2PredefinedFilterType",
    "V2PredefinedLookupType",
    "V2ProcTxRcvPolarizationType",
    "V2ProcessingEventType",
    "V2ProcessingModuleType",
    "V2ProcessorInformationType",
    "V2ProductClassificationType",
    "V2ProductCreationType",
    "V2ProductDisplayType",
    "V2ProductGenerationOptionsType",
    "V2ProductPlaneType",
    "V2ProductProcessingType",
    "V2RRDSType",
    "V2RangeAdjustmentAlgorithmType",
    "V2RemapChoiceType",
    "V2RenderingIntentType",
    "V2SIDD",
    "V2ScalingType",
    "V2ShadowDirectionType",
    "V2SharpnessEnhancementType",
    "V2TxRcvPolarizationType",
    "V2ValidDataType",
    "V3AccuracyType",
    "V3AcheivedResolutionType",
    "V3AnnotationObjectType",
    "V3AnnotationType",
    "V3AnnotationsType",
    "V3BandEqualizationType",
    "V3BankCustomType",
    "V3BaseProjectionType",
    "V3ClassificationGuidanceType",
    "V3ColorDisplayRemapType",
    "V3ColorManagementModuleType",
    "V3ColorSpaceTransformType",
    "V3CompressionType",
    "V3CustomLookupType",
    "V3CylindricalProjectionType",
    "V3DRAHistogramOverridesType",
    "V3DRAOverrides",
    "V3DRAParameters",
    "V3DigitalElevationDataType",
    "V3DownsamplingMethodType",
    "V3DownstreamReprocessingType",
    "V3DynamicRangeAdjustmentType",
    "V3EarthModelType",
    "V3EqualizationAlgorithmType",
    "V3ExploitationFeaturesCollectionGeometryType",
    "V3ExploitationFeaturesCollectionInformationType",
    "V3ExploitationFeaturesCollectionPhenomenologyType",
    "V3ExploitationFeaturesCollectionType",
    "V3ExploitationFeaturesProductType",
    "V3ExploitationFeaturesType",
    "V3FilterBankCoefType",
    "V3FilterBankType",
    "V3FilterDatabaseNameType",
    "V3FilterKernelCoefType",
    "V3FilterKernelType",
    "V3FilterOperationType",
    "V3FilterType",
    "V3GeoDataType",
    "V3GeographicCoordinatesType",
    "V3GeographicProjectionType",
    "V3GeometricChipType",
    "V3GeometricTransformType",
    "V3GeopositioningType",
    "V3GeopositioningTypeCoordinateSystemType",
    "V3GeopositioningTypeGeodeticDatum",
    "V3GeopositioningTypeReferenceEllipsoid",
    "V3GeopositioningTypeSoundingDatum",
    "V3GeopositioningTypeVerticalDatum",
    "V3ImageCornersType",
    "V3InputROIType",
    "V3InteractiveProcessingType",
    "V3J2KSubtype",
    "V3J2KType",
    "V3KernelCustomType",
    "V3LUTInfoType",
    "V3LayerInfoType",
    "V3LayerType",
    "V3Lookup3TableType",
    "V3LookupTableType",
    "V3MeasurableProjectionType",
    "V3MeasurementType",
    "V3MeasurementTypeARPFlag",
    "V3MonochromeDisplayRemapType",
    "V3NewLookupTableType",
    "V3NonInteractiveProcessingType",
    "V3Orientation",
    "V3PixelType",
    "V3PlaneProjectionType",
    "V3PolygonType",
    "V3PolynomialProjectionType",
    "V3PositionalAccuracyType",
    "V3PredefinedFilterType",
    "V3PredefinedLookupType",
    "V3ProcTxRcvPolarizationType",
    "V3ProcessingEventType",
    "V3ProcessingModuleType",
    "V3ProcessorInformationType",
    "V3ProductClassificationType",
    "V3ProductCreationType",
    "V3ProductDisplayType",
    "V3ProductGenerationOptionsType",
    "V3ProductPlaneType",
    "V3ProductProcessingType",
    "V3RRDSType",
    "V3RangeAdjustmentAlgorithmType",
    "V3RemapChoiceType",
    "V3RenderingIntentType",
    "V3SIDD",
    "V3ScalingType",
    "V3ShadowDirectionType",
    "V3SharpnessEnhancementType",
    "V3TxRcvPolarizationType",
    "V3ValidDataType",
]



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:ism-cvenum"


class CVEnumISMNonICValues(Enum):
    """(U) All currently valid Non-IC markings from the published register
    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMNonIC.xml

    :cvar SC: SPECIAL CATEGORY
    :cvar SINFO: SENSITIVE INFORMATION
    :cvar DS: LIMITED DISTRIBUTION
    :cvar XD: EXCLUSIVE DISTRIBUTION
    :cvar ND: NO DISTRIBUTION
    :cvar SBU: SENSITIVE BUT UNCLASSIFIED
    :cvar SBU_NF: SENSITIVE BUT UNCLASSIFIED NOFORN
    :cvar LES: LAW ENFORCEMENT SENSITIVE
    :cvar LES_NF: LAW ENFORCEMENT SENSITIVE NOFORN
    """

    SC = "SC"
    SINFO = "SINFO"
    DS = "DS"
    XD = "XD"
    ND = "ND"
    SBU = "SBU"
    SBU_NF = "SBU-NF"
    LES = "LES"
    LES_NF = "LES-NF"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:cvenum:ism:complieswith"


class CVEnumISMCompliesWithValues(Enum):
    """(U) ISM rule sets documents may comply with.

    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMCompliesWith.xml

    :cvar USGOV: Document claims compliance with all rules encoded in ISM for documents produced by the US
        Federal Government. This is the minimum set of rules for US documents to adhere to, and all US documents
        should claim compliance with USGov. For example, a US Intelligence Community document should claim
        ism:compliesWith="USGov USIC".
    :cvar USIC: Document claims compliance with all rules encoded in ISM for documents produced by the US
        Intelligence Community. Documents that claim compliance with USIC MUST also claim compliance with USGov.
    :cvar USDOD: Document claims compliance with all rules encoded in ISM for documents produced by the US
        Department of Defense. Documents that claim compliance with USDOD MUST also claim compliance with USGov.
    :cvar OTHER_AUTHORITY: Document claims compliance with an authority other than the USGov, USIC, or USDOD.
        This token is not allowed if the ism:ownerProducer contains USA.
    """

    USGOV = "USGov"
    USIC = "USIC"
    USDOD = "USDOD"
    OTHER_AUTHORITY = "OtherAuthority"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

from enum import Enum


class GDALCompressionOptions(str, Enum):
    """
    Enumeration defining compression algorithms for image.
    """

    NONE = "NONE"
    JPEG = "JPEG"
    J2K = "J2K"
    LZW = "LZW"


class GDALImageFormats(str, Enum):
    """
    Subset of GDAL supported image formats commonly used by this software. See
    https://gdal.org/drivers/raster/index.html for a complete listing of the formats.
    """

    NITF = "NITF"
    JPEG = "JPEG"
    PNG = "PNG"
    GTIFF = "GTiff"


class RangeAdjustmentType(str, Enum):
    """
    Enumeration defining ways to scale raw image pixels to an output value range.

    - NONE indicates that the full range available to the input type will be used.
    - MINMAX chooses the portion of the input range that actually contains values.
    - DRA is a dynamic range adjustment that attempts to select the most important portion of the input range.
      It differs from MINMAX in that it can exclude outliers to reduce the impact of unusually bright/dark
      spots in an image.
    """

    NONE = "NONE"
    MINMAX = "MINMAX"
    DRA = "DRA"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:cvenum:ism:scicontrols"


class CVEnumISMSCIControlsValuesvalue(Enum):
    """
    :cvar EL: ENDSEAL
    :cvar EL_EU: ECRU
    :cvar EL_NK: NONBOOK
    :cvar HCS: HCS
    :cvar HCS_O: HCS-O
    :cvar HCS_P: HCS-P
    :cvar KDK: KLONDIKE
    :cvar KDK_BLFH: KDK BLUEFISH
    :cvar KDK_IDIT: KDK IDITAROD
    :cvar KDK_KAND: KDK KANDIK
    :cvar RSV: RESERVE
    :cvar SI: SPECIAL INTELLIGENCE
    :cvar SI_G: SI-GAMMA
    :cvar TK: TALENT KEYHOLE
    """

    EL = "EL"
    EL_EU = "EL-EU"
    EL_NK = "EL-NK"
    HCS = "HCS"
    HCS_O = "HCS-O"
    HCS_P = "HCS-P"
    KDK = "KDK"
    KDK_BLFH = "KDK-BLFH"
    KDK_IDIT = "KDK-IDIT"
    KDK_KAND = "KDK-KAND"
    RSV = "RSV"
    SI = "SI"
    SI_G = "SI-G"
    TK = "TK"
